// file: attribute_node.h
//
 
// make sure definitions are made only once
//
#ifndef __ISIP_ATTRIBUTE_NODE
#define __ISIP_ATTRIBUTE_NODE
 
// isip include files
//
#include <integral.h>

// Attribute_node: a class that describe an attribute node.
//
class Attribute_node {

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // declare a variable for debugging
  //
  int_4 debug_level_d;

  // the name of the attribute
  //
  char_1* label_d;

  // the nature type of the attribute (discrete or continuous)
  //
  int_4 type_d;

  // the nature ranging of the attribute (auto ranging, manual
  // ranging, or enumerated type)
  //
  int_4 range_d;

  // the range values (only applicable when ranging type is auto or
  // manual)
  //
  float_4 min_d;
  float_4 max_d;

  // the number of values in case of discrete type
  //
  int_4 num_values_d;

  // the list of values in case of discrete type
  //
  char_1** values_d;
  
  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // required methods
  //
  char_1* name_cc();
  volatile void error_handler_cc(char_1* method_name, char_1* message);
  logical_1 debug_cc(FILE* fp, char_1* message);
  logical_1 set_debug_cc(int_4 debug_level);
  
  // destructors/constructors
  //
  ~Attribute_node();
  Attribute_node();
  Attribute_node(char_1* label, int_4 type, int_4 range,
		 int_4 num_values, char_1** values);
  Attribute_node(char_1* label, int_4 type, int_4 range,
		 float_4 min, float_4 max,
		 int_4 num_values, char_1** values);
  Attribute_node(char_1* label, int_4 type, int_4 range,
		 float_4 min, float_4 max);

  // method to set attribute information: name, type, range, min, max,
  // num_values, and values
  //
  logical_1 set_label_cc(char_1* name);
  logical_1 set_type_cc(int_4& type);
  logical_1 set_range_cc(int_4& range);
  logical_1 set_min_cc(float_4& min);
  logical_1 set_max_cc(float_4& max);
  logical_1 set_num_values_cc(int_4& num_values);
  logical_1 set_values_cc(char_1** values);

  // method to get attribute information: name, type, range, min, max,
  // num_values, and values
  //
  logical_1 get_label_cc(char_1* name);
  logical_1 get_type_cc(int_4& type);
  logical_1 get_range_cc(int_4& range);
  logical_1 get_min_cc(float_4& min);
  logical_1 get_max_cc(float_4& max);
  logical_1 get_num_values_cc(int_4& num_values);
  logical_1 get_values_cc(char_1** values);

  // method to check for errors in the node
  //
  logical_1 check_errors_cc();
  logical_1 check_errors_cc(FILE* fp);
  
  //---------------------------------------------------------------------------
  //
  // private members
  //
  //---------------------------------------------------------------------------
private:

  //
  // end of class
  
};

//
// end of include file
#endif
