// file: $DT/class/decision_tree/v1.0/dt_anode_1.cc
//

// isip include files
//
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>
#include <attribute_node.h>
#include <attribute_node_constants.h>
#include <linked_list.h>
#include <string.h>

// method: generate_attr_node_cc
//
// arguments:
//  char_1* label_a (input) the label of the class
//  char_1* value_a (input) the value of the class
//
// return: a logical_1 indicating status
//
logical_1 Decision_tree::generate_attr_node_cc (char_1* label_a,
						char_1* value_a) {
  
  // argument validity check
  //
  if (label_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"generate_attr_node_cc",
                     (char_1*)"attribute label is not valid");
  }

  if (value_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"generate_attr_node_cc",
                     (char_1*)"attribute value is not valid");
  }

  // declare some useful variables to insert in the node
  //
  int_4 type_a = ATTR_TYPE_NONE;
  int_4 range_a = ATTR_RANGE_NONE;
  float_4 min_a=0;
  float_4 max_a=0;
  char_1** attr_values_a = new char_1*[DT_MAX_NUM_OF_ATTR_VALUES];
  int_4 num_attr_values_a;
  Attribute_node* node_a = (Attribute_node*)NULL;

  // check type and range of the attribute
  //
  find_type_cc(value_a, type_a);
  find_range_cc(value_a, type_a, range_a, min_a, max_a);

  // check for each discrete items and ranging method if the attribute
  // type is discrete
  //
  if (type_a == ATTR_TYPE_DISCRETE) {
    if (range_a == ATTR_RANGE_MANUAL) {
      num_attr_values_a = (int_4) (max_a - min_a + 1);
      manual_range_cc(attr_values_a, (int_4) min_a, (int_4) max_a);
    }
    
    // for enumerated items, we tokenize the value by ',' and store
    // them as an array
    //
    else {
      pre_tokenize_cc(attr_values_a, num_attr_values_a, value_a, ',');
    }
  }

  // create an attribute node according to the type and range of the
  // node
  //
  if (type_a == ATTR_TYPE_CONTINUOUS) {
    node_a = new Attribute_node(label_a, type_a, range_a, min_a, max_a);
  }
  else if (range_a == ATTR_RANGE_ENUM) {
    node_a = new Attribute_node(label_a, type_a, range_a,
				num_attr_values_a, attr_values_a);
  }
  else {
    node_a = new Attribute_node(label_a, type_a, range_a, min_a, max_a,
				num_attr_values_a, attr_values_a);
  }

  // insert the node into the attribute list
  //
  attribute_list_d -> insert_cc ((void_p)node_a);

  // clean up, the elements in attr_values_a are not deleted because
  // they are just pointers to different places in value_a and the
  // memory was not allocated.  When value_a is deleted, all elements
  // in attr_values_a should be deleted.
  //
  if (attr_values_a != (char_1**)NULL) {
    delete [] attr_values_a;
    attr_values_a = (char_1**)NULL;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


