// file: $DT/class/tree/v1.0/dt_bayes_0.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: discrete_bayes_split_cc
//
// arguments: 
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::discrete_bayes_split_cc(int_4* max_mask_a,
					       Attribute_node* attr_node_a,
 					       Tree_node* tree_node_a) {

  // get the label for this attribute
  //
  char_1* attr_label = new char_1[ISIP_MAX_STRING_LENGTH];
  if (attr_node_a->get_label_cc(attr_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"discrete_bayes_split_cc",
		     (char_1*)"unable to get the label for this attribute");
  }

  // get the number of values for this attribute
  //
  int_4 values_per_attr;
  if (attr_node_a->get_num_values_cc(values_per_attr) != ISIP_TRUE) {
    error_handler_cc((char_1*)"discrete_bayes_split_cc",
		     (char_1*)"unable to get the number of attributes");
  }

  // get the actual values for this attribute
  //
  char_1** attr_values = new char_1*[values_per_attr];
  for (int_4 i=0; i<values_per_attr; i++) {
    attr_values[i] = new char_1 [ISIP_MAX_STRING_LENGTH];
  }
  if (attr_node_a->get_values_cc(attr_values) != ISIP_TRUE) {
    error_handler_cc((char_1*)"discrete_bayes_split_cc",
		     (char_1*)"unable to get the values of attributes");
  }
  
  // calculate the number of possible permutations of the categories
  //
  int_4 iter;
  iter = (int_4)pow(2, values_per_attr-1);

  // create space for a temporary mask
  //
  int_4* mask = new int_4[values_per_attr];

  // maximum probability
  //
  float_4 max_prob = -10000;
  
  // loop through all possible permutations
  //
  for (int_4 i=1; i<iter; i++) {

    // create masks of 0, 1 for each possible permutation
    //
    permute_cc(i, mask, values_per_attr);

    float_4 pl = calculate_bayes_left_prob_cc(mask, values_per_attr, attr_values,
					      attr_label, tree_node_a);
    float_4 pr = calculate_bayes_right_prob_cc(mask, values_per_attr, attr_values,
					       attr_label, tree_node_a);
    
    // keep the maximum split
    //
    float_4 prob = pl + pr;
    
    if (prob > max_prob ) {
      max_prob = prob;
      for (int_4 k=0; k<values_per_attr; k++ ) {
	max_mask_a[k] = mask[k];
      }
    }
  }

  // delete scratch space
  //
  delete [] mask;
  for (int_4 i=0; i<values_per_attr; i++) {
    delete [] attr_values[i];
  }
  delete [] attr_label;
  
  // return maximum value 
  //
  return max_prob;
}


















