// file: $DT/class/tree/v1.0/dt_bayes_1.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: continuous_entropy_split_cc
//
// arguments: 
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::continuous_bayes_split_cc(float_4& max_split_a,
						 Attribute_node* attr_node_a,
						 Tree_node* tree_node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (tree_node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get data nodes from in tree node");
  }

  // get the number of data nodes in the linked list
  //
  int_4 num_data_nodes = 0;
  if (tree_node_a->get_num_data_nodes_cc(num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get number of data nodes in tree node");
  }
  
  if (num_data_nodes <= 0) {
    error_handler_cc((char_1*)"continous_bayes_split",
		     (char_1*)"num_data_nodes <= 0");
  }

  // get the attribute value of this attribute for each data node
  //
  char_1* attr_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  if (attr_node_a->get_label_cc(attr_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split",
		     (char_1*)"unable to get attribute label");
  }
  float_4* attr_values = new float_4 [num_data_nodes];

  // loop over all data nodes
  //
  for (int_4 i=0; i<num_data_nodes; i++) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();
    if (data_node == (Data_node*)NULL) {
      error_handler_cc((char_1*)"continous_bayes_split",
		       (char_1*)"NULL data node");
    }
    
    if (data_node->get_value_cc(attr_values[i], attr_label) != ISIP_TRUE) {
      error_handler_cc((char_1*)"continous_bayes_split",
		       (char_1*)"unable to get attribute value");
    }
    data_list = data_list->get_next_cc();
  }
  
  // sort the attribute values
  //
  int_4* index = new int_4 [num_data_nodes];
  if (qsort_cc(index, attr_values, num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to sort data");
  }

  // loop through all possible splits for this attribute
  //
  int_4 num_classes;
  class_node_d->get_num_values_cc(num_classes);

  float_4 max_prob = -10000;
  
  for (int_4 i=0; i<num_data_nodes-1; i++) {
    if (attr_values[index[i]] != attr_values[index[i+1]] ) {
      float_4 split = (attr_values[i] + attr_values[i+1])/2;
      float_4 pl = calculate_bayes_left_prob_cc(split, tree_node_a);
      float_4 pr = calculate_bayes_right_prob_cc(split, tree_node_a);
      // keep the maximum split
      //
      float_4 prob = pl + pr;
      
      if (prob > max_prob) {
	max_prob = prob;
	max_split_a = split;
      }
    }
  }
  
  // delete scratch space
  //
  delete [] attr_values;
  delete [] index;
  if (attr_label != (char_1*)NULL) {
    delete [] attr_label;
  }
  
  // return maximum value 
  //
  return max_prob;
}











