// file: $DT/class/tree/v1.0/dt_bayes_2.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: calculate_bayes_left_prob_cc
//
// arguments: 
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::calculate_bayes_left_prob_cc (float_4 split_a,
						     Tree_node* tree_node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (tree_node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get data nodes from in tree node");
  }

  // get the class label and the number of classes
  //
  char_1* class_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  int_4 num_classes = 0;
  if (class_node_d->get_label_cc(class_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		     (char_1*)"unable get class label");
  }

  if (class_node_d->get_num_values_cc(num_classes) != ISIP_TRUE) {
     error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		     (char_1*)"unable get number of classes");
  }

  // keep a count of how many instances of each class on the left tree
  //
  int_4* class_count = new int_4 [num_classes];
  for (int_4 i=0; i<num_classes; i++) {
    class_count[i] = 0;
  }
  
  // keep a count of how many instances on the left tree
  //
  int_4 left_count = 0;
  
  float_4 attr_value;
  char_1* class_value = new char_1 [ISIP_MAX_STRING_LENGTH];
  while (data_list != (Linked_list*)NULL) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();
    
    if (data_node->get_value_cc(attr_value, class_label) != ISIP_TRUE) {
      error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		       (char_1*)"unable get attribute value");
    }

    // continuous value so we can do straight math comparison
    //
    if (attr_value < split_a) {
      if (data_node->get_item_cc(class_value, class_label) != ISIP_TRUE) {
	error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		       (char_1*)"unable get class value");
      }
      left_count++;
    }
    for (int_4 i=0; i<num_classes; i++) {
      if (strcmp((char*)class_value, (char*)class_names_d[i]) == 0) {
	class_count[i]++;
	break;
      }
    }
    data_list = data_list->get_next_cc();
  }
  
  // calculate the left probability
  //

  // some samples are in the left tree
  //
  float_4 left_prob = 0;
  if (left_count > 0) {
    for (int_4 i=0; i<num_classes; i++) {
      if (class_count[i] > 0) {
	left_prob += log((float_4)class_count[i]/left_count)*class_count[i];
      }
    }
  }

  // delete scratch space
  //
  if (class_label != (char_1*)NULL) {
    delete [] class_label;
  }
  delete [] class_count;
  delete [] class_value;
  
  // return maximum value 
  //
  return left_prob;
}


















