// file: $DT/class/tree/v1.0/dt_bayes_4.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: calculate_bayes_left_prob_cc
//
// arguments: 
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::calculate_bayes_left_prob_cc (int_4* mask_a,
						     int_4 values_per_attr_a,
						     char_1** attr_values_a,
						     char_1* attr_label_a,
						     Tree_node* tree_node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (tree_node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get data nodes from in tree node");
  }

  // get the class label and the number of classes
  //
  char_1* class_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  int_4 num_classes = 0;

  if (class_node_d->get_label_cc(class_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		     (char_1*)"unable get class label");
  }

  if (class_node_d->get_num_values_cc(num_classes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		     (char_1*)"unable get number of classes");
  }
  
  // keep a count of how many samples on the left tree
  //
  int_4 left_count = 0;

  // keep a count of how many samples of each class on the left tree
  //
  int_4* class_count = new int_4 [num_classes];
  for (int_4 i=0; i<num_classes; i++) {
    class_count[i] = 0;
  }
    
  char_1* class_value = new char_1 [ISIP_MAX_STRING_LENGTH];  
  char_1* attr_value = new char_1 [ISIP_MAX_STRING_LENGTH];

  // loop over all the data nodes
  //
  while (data_list != (Linked_list*)NULL) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();

    // get the class value for this node
    //
    if (data_node->get_item_cc(class_value, class_label) != ISIP_TRUE) {
      error_handler_cc((char_1*)"continous_bayes_split_cc",
		       (char_1*)"unable to get class value");
    }

    // get the attribute value of this attribute label for this node
    //
    if (data_node->get_item_cc(attr_value, attr_label_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		       (char_1*)"unable get attribute value");
    }

    for (int_4 i=0; i<values_per_attr_a; i++) {
      if ((strcmp((char*)attr_value, (char*)attr_values_a[i]) == 0) &&
	  (mask_a[i] == DT_SPLIT_LEFT)) {
	left_count++;
	for (int_4 j=0; j<num_classes; j++) {
	  if (strcmp((char*)class_value, (char*)class_names_d[j]) == 0) {
	    class_count[j]++;
	  }
	}
      }
    }
    data_list = data_list->get_next_cc();
  }
  
  // calculate the left probability
  //

  // some samples are in the left tree
  //
  float_4 left_prob = 0;
  if (left_count > 0) {
    for (int_4 i=0; i<num_classes; i++) {
      if (class_count[i] > 0) {
	left_prob += log((float_4)class_count[i]/left_count)*class_count[i];
      }
    }
  }
  
  // delete scratch space
  //
  delete [] class_label;
  delete [] class_count;
  delete [] class_value;
  delete [] attr_value;
  
  // return maximum value 
  //
  return left_prob;
}

