// file: $DT/class/tree/v1.0/dt_cal_0.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: discrete_bayes_split_cc
//
// arguments: 
//
// return: logical_1 indicating success
//
// this method calculates the class prior probabilities from the training
// data.
//
logical_1 Decision_tree::calculate_prior_prob_cc() {

  // get the label of the classes
  //
  char_1* class_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  class_node_d->get_label_cc(class_label);

  // get the name of each class
  //
  num_classes_d = 0;
  class_node_d->get_num_values_cc(num_classes_d);
  if (num_classes_d <= 0) {
    error_handler_cc((char_1*)"calculate_prior_prob_cc",
		     (char_1*)"number of classes is zero");
  }
  int_4* class_count = new int_4 [num_classes_d];

  // check to see if somewhere else already allocated memory for
  // class_names_d.  If so, then there is some problem, error.
  //
  if (class_names_d != (char_1**)NULL) {
    error_handler_cc((char_1*)"calculate_prior_prob_cc",
		     (char_1*)"class_names_d is not NULL");
  }
  // allocate space for each class name and initialize each class count
  //
  class_names_d = new char_1* [num_classes_d];
  for (int_4 i=0; i<num_classes_d; i++) {
    class_names_d[i] = new char_1 [ISIP_MAX_STRING_LENGTH];
    class_count[i] = 0;
  }
  class_node_d->get_values_cc(class_names_d);

  // now go into each data node and sum up the occurrences of each class,
  //
  Linked_list* tmp_list = data_list_d;
  char_1 class_value[ISIP_MAX_STRING_LENGTH];
  int_4 total_count = 0;
  while (tmp_list != (Linked_list*)NULL) {
    Data_node* data_node = (Data_node*)tmp_list->get_node_cc();
    data_node->get_item_cc(class_value, class_label);
    for (int_4 i=0; i<num_classes_d; i++) {
      if (strcmp((char*)class_value, (char*)class_names_d[i]) == 0) {
	class_count[i]++;
      }
    }
    total_count++;
    tmp_list = tmp_list->get_next_cc();
  }

  // check to see if somewhere else already assigned
  // class_prior_probs_d.  If so, then there is some problem, error.
  //
  if (class_prior_probs_d != (float_4*)NULL) {
    error_handler_cc((char_1*)"calculate_prior_prob_cc",
		     (char_1*)"class_prior_probs_d is not NULL");
  }
  class_prior_probs_d = new float_4 [num_classes_d];

  // calculate the prior probabilities of the classes
  //
  for (int_4 i=0; i<num_classes_d; i++) {
    class_prior_probs_d[i] = (float_4)class_count[i]/total_count;
  }

  // delete scratch space
  //
  delete [] class_count;
  if (class_label != (char_1*)NULL) {
    delete [] class_label;
  }
  
  return ISIP_TRUE;
}


















