// file: $DT/class/decision_tree/v1.0/dt_cflmt_0.cc
//

// isip include files
//
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: confidence_limit_cc
//
// arguments:
//  int_4 num_of_population (input): the total number of population
//  int_4 num_of_errors (input): the total number of errors
//
// return: a float_4 value consisting of the upper confidence limit using
//         the given number of population and error.  The confidence
//         level is defined in the constant header file
//
float_4 Decision_tree::confidence_limit_cc (int_4 population_a,
					    int_4 errors_a) {
  
  float_4  value;
  static float_4 val[] = {0, 0.001, 0.005, 0.01, 0.05, 0.10, 0.20, 0.40, 1.00};
  static float_4 dev[] = {100, 3.09, 2.58, 2.33, 1.65, 1.28, 0.84, 0.25, 0.00};
  static float_4 coeff = 0;
  
  // check arguments
  //
  if (errors_a > population_a) {
    error_handler_cc((char_1*)"confidence_limit_cc",
		     (char_1*)"number of error samples can't be greater than the entire population");
  }
  
  if (!coeff) {
    int_4 i = 0;
    
    while (DT_CONFIDENCE_LIMIT > val[i]) i++;
    coeff = dev[i-1] + (dev[i] - dev[i-1]) * (DT_CONFIDENCE_LIMIT - val[i-1])
      / (val[i] - val[i-1]);
    coeff = coeff * coeff;
  }
  
  // calculate the value when the number of error is zero
  //
  if ((float_4)errors_a < ISIP_MINIMUM_FLOAT_4) {
    return (population_a * (1 - exp (log (DT_CONFIDENCE_LIMIT)
				     / population_a)));
  }
  
  // calculate the value when the number of error is less than 1
  //
  if ((float_4)errors_a < 0.99999) {
    value = population_a * (1 - exp (log (DT_CONFIDENCE_LIMIT)
				     / population_a));
    return (value + errors_a * (confidence_limit_cc(population_a, 1) - value));
  }
  else if ((float_4)errors_a + 0.5 >= population_a) {
    return 0.67 * (population_a - errors_a);
  }
  else {
    value = (errors_a + 0.5 + coeff / 2
	     + sqrt (coeff * ((errors_a + 0.5)
			      * (1 - ( errors_a + 0.5)  / population_a)
			      + coeff / 4))) / (population_a + coeff);
    return (population_a * value - errors_a);
  }
}


