// file: $DT/class/decision_tree/v1.0/dt_cnode_0.cc
//

// isip include files
//
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>
#include <attribute_node.h>
#include <attribute_node_constants.h>
#include <string.h>

// method: generate_class_node_cc
//
// arguments:
//  char_1* label_a (input) the label of the class
//  char_1* value_a (input) the value of the class
//
// return: a logical_1 indicating status
//
logical_1 Decision_tree::generate_class_node_cc (char_1* label_a,
						 char_1* value_a) {
  
  // argument validity check
  //
  if (label_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"generate_class_node_cc",
                     (char_1*)"class label is not valid");
  }

  if (value_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"generate_class_node_cc",
                     (char_1*)"class value is not valid");
  }

  // declare some useful variables to insert in the node
  //
  int_4 type_a = ATTR_TYPE_NONE;
  int_4 range_a = ATTR_RANGE_NONE;
  float_4 min_a=0;
  float_4 max_a=0;
  char_1** class_values_a = new char_1*[DT_MAX_NUM_OF_ATTR_VALUES];
  int_4 num_class_values_a;

  // check type and range of the class
  //
  find_type_cc(value_a, type_a);
  find_range_cc(value_a, type_a, range_a, min_a, max_a);

  // error out if class node is continuous
  //
  if (type_a == ATTR_TYPE_CONTINUOUS) {
    error_handler_cc((char_1*)"generate_class_node_cc",
		     (char_1*)"cannot have continuous value for class node");
  }

  else {

    // for manual range items, we find the number of values, generate
    // an element for each of the discrete value, and create the class
    // node
    //
    if (range_a == ATTR_RANGE_MANUAL) {
      num_class_values_a = (int_4) (max_a - min_a + 1);
      manual_range_cc(class_values_a, (int_4) min_a, (int_4) max_a);
      class_node_d = new Attribute_node (label_a, type_a, range_a,
					 min_a, max_a, num_class_values_a,
					 class_values_a);
    }
    
    // for enumerated items, we tokenize the value by ',' and store
    // them as an array
    //
    else {
      pre_tokenize_cc(class_values_a, num_class_values_a, value_a, ',');
      class_node_d = new Attribute_node (label_a, type_a, range_a,
					 num_class_values_a,
					 class_values_a);
    }
  }

  // clean up, the elements in class_values_a are not deleted because
  // they are just pointers to different places in value_a and the
  // memory was not allocated.  When value_a is deleted, all elements
  // in class_values_a should be deleted.
  //
  if (class_values_a != (char_1**)NULL) {
    delete [] class_values_a;
    class_values_a = (char_1**)NULL;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


