// file: $DT/class/tree/v1.0/dt_entr_1.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: continuous_entropy_split_cc
//
// arguments: 
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::continuous_entropy_split_cc(float_4& max_split_a,
						   Attribute_node* attr_node_a,
						   Tree_node* node_a) {

  float_4 split_a=0;
  float_4 max_a=0;
  float_4 pl;                         // left node probability
  float_4 pj_l;                       // prob class j given left node
  float_4 pj_r;                       // prob class j given right node
  float_4 sum;
  float_4 delta_i;                    // twoing criterion
  int_4 num_classes_a;
  int_4* class_given_left;
  int_4* class_given_right;
  int_4 count_left=0, count_right=0;
  
  class_node_d->get_num_values_cc(num_classes_a);

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_entropy_split_cc",
		     (char_1*)"unable to get data nodes from in tree node");
  }

  // get the number of data nodes in the linked list
  //
  int_4 num_data_nodes = 0;
  if (node_a->get_num_data_nodes_cc(num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_entropy_split_cc",
		     (char_1*)"unable to get number of data nodes in tree node");
  }
  
  if (num_data_nodes <= 0) {
    error_handler_cc((char_1*)"continous_entropy_split",
		     (char_1*)"num_data_nodes <= 0");
  }

  // get the attribute value for each data node
  //
  float_4* attr_values = new float_4 [num_data_nodes];
  char_1* attr_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  if (attr_node_a->get_label_cc(attr_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_entropy_split",
		     (char_1*)"unable to get attribute label");
  }
  
  for (int_4 i=0; i<num_data_nodes; i++) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();
    if (data_node == (Data_node*)NULL) {
      error_handler_cc((char_1*)"continous_entropy_split",
		       (char_1*)"NULL data node");
    }

    if (data_node->get_value_cc(attr_values[i], attr_label) != ISIP_TRUE) {
      error_handler_cc((char_1*)"continous_entropy_split",
		       (char_1*)"unable to get attribute value");
    }
    data_list=data_list->get_next_cc();
  }
  
  // sort the attribute values
  //
  int_4* index = new int_4 [num_data_nodes];
  if (qsort_cc(index, attr_values, num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to sort data");
  }

  class_given_left = new int_4[num_classes_a];
  class_given_right = new int_4[num_classes_a];
  
  // loop through all possible splits
  //
  for (int_4 i=0; i<num_data_nodes-1; i++) {
    split_a = (attr_values[index[i]] + attr_values[index[i+1]])/2;
    
    for ( int_4 j=0; j<num_classes_a; j++ ){
      class_given_left[j]=0;
      class_given_right[j]=0;
    }
    
    pl=calculate_entropy_prob_cc(split_a,attr_node_a,node_a,
				 class_given_left, class_given_right,
				 count_left, count_right);
    sum=0;

    // get the sum of the absolute values of the left node given a class
    // minus the right node given a class for all possible classes
    //
    for ( int_4 j=0; j<num_classes_a; j++ ) {
      float_4 plpr;
      pj_l= (float_4)class_given_left[j]/(float_4)count_left;
      pj_r= (float_4)class_given_right[j]/(float_4)count_right;
      plpr = pj_l - pj_r;
      if (plpr < 0) plpr=-plpr;
      sum+= plpr;
    }

    // compute the twoing criterion for this split
    //
    delta_i = pl*(1-pl)*(pow(sum,2))/4;

    // keep the maximum split
    //
    if ( delta_i > max_a ) {
      max_a = delta_i;
      max_split_a = split_a;
    }
  }

  delete [] class_given_left;
  delete [] class_given_right;

  // delete scratch space
  //
  delete [] attr_values;
  delete [] index;
  if (attr_label != (char_1*)NULL) {
    delete [] attr_label;
  }

  // return maximum value of the twoing criterion
  //
  return max_a;
}











