// file: $DT/class/decision_tree/v1.0/dt_find_1.cc
//

// isip include files
//
#include "decision_tree.h"
#include "attribute_node_constants.h"
#include <integral_constants.h>
#include <string.h>

// method: find_range_cc
//
// arguments:
//  char_1** attr_valuelist_a (input) the attribute information
//  int_4    type_a (input) the attribute type
//  int_4&   range_a (output) the attribute range
//  float_4& min_a (output) the minimum value of the range
//  float_4& max_a (output) the maximum value of the range
//
// return: a logical_1 indicating status
//
logical_1 Decision_tree::find_range_cc (char_1* attr_valuelist_a,
					int_4 type_a, int_4& range_a,
					float_4& min_a, float_4& max_a) {
  
  // argument validity check
  //
  if (attr_valuelist_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"find_range_cc",
		     (char_1*)"can't find range of a null pointer");
  }

  if (strstr ((char*)attr_valuelist_a,
	      (char*)ATTR_RANGE_MANUAL_FLAG) == NULL) {
    if (type_a == ATTR_TYPE_CONTINUOUS) {
      range_a = ATTR_RANGE_AUTO;
    }
    else if (type_a == ATTR_TYPE_DISCRETE) {
      range_a = ATTR_RANGE_ENUM;
    }
    else {
      error_handler_cc((char_1*)"find_range_cc",
		       (char_1*)"can't find range of unknown type");
    }
  }
  else {
    range_a = ATTR_RANGE_MANUAL;
    char_1* tmp_buf = new char_1[ISIP_MAX_STRING_LENGTH];
    if (type_a == ATTR_TYPE_CONTINUOUS) {
      sscanf ((char*)attr_valuelist_a, "cont%s", (char*)tmp_buf);
      find_minmax_cc(tmp_buf, min_a, max_a);
    }
    else if (type_a == ATTR_TYPE_DISCRETE) {
      strcpy((char*)tmp_buf, (char*)attr_valuelist_a);
      find_minmax_cc(tmp_buf, min_a, max_a);
    }
    else {
      error_handler_cc((char_1*)"find_range_cc",
		       (char_1*)"can't find range for unknown type");
    }
    if (tmp_buf != (char_1*)NULL) {
      delete [] tmp_buf;
    }
    
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


