// file: $DT/class/tree/v1.0/dt_gain_0.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: discrete_gainratio_split_cc
//
// arguments: 
//  int_4* max_mask (input): the maximum mask
//  int_4  num_categories (input): the number of categories
//  Attribute_node* attr_node (input): the attribute node we are
//                                     interested in
//  Tree_node* node (input): the tree node we are currently trying to split
//
// return: float_4 indicating the gain ratio
//
// this method performs gain ratio spliting rule for discrete values
// and returns the value associated with the gain ratio
//
float_4 Decision_tree::discrete_gainratio_split_cc(int_4* max_mask_a,
						   int_4 num_categories_a,
						   Attribute_node* attr_node_a,
						   Tree_node* node_a) {

  float_4 total_info;
  float_4 info;
  float_4 gain;
  float_4 split_info;
  float_4 gain_ratio=0;
  int_4 iter_a;
  float_4 max_a=-1.0e4;
  int_4* mask_a = new int_4[num_categories_a];
  int_4 num_classes_a;
  int_4 num_data;
  int_4 left=0, right=0;

  class_node_d->get_num_values_cc(num_classes_a);

  total_info = calculate_node_info_cc(num_classes_a, node_a);
  
  // number of possible permutations of the categories
  //
  iter_a = (int_4) pow(2,(num_categories_a-1));

  // loop through all possible permutations
  //
  for (int_4 i=1;i<iter_a;i++) {

    // create masks of 0,1 for each possible permutation
    //
    permute_cc(i, mask_a, num_categories_a);

    info = calculate_info_cc(mask_a, num_classes_a, num_categories_a,
			     attr_node_a, node_a, num_data, left, right);

    gain = total_info - info;
    split_info = calculate_split_info_cc(left, right, num_data);
    if (split_info == 0) {
      if (total_info == info) {
	gain_ratio = 0;
      }
      else {
	gain_ratio = max_a;
      }
    }
    else {
      gain_ratio = gain / split_info;
    }
    
    // keep the maximum split
    //
    if ( gain_ratio > max_a ) {
      max_a = gain_ratio;
      for ( int_4 k=0; k<num_categories_a; k++ ) max_mask_a[k] = mask_a[k];
    }
  }

  delete [] mask_a;
    
  // return max value of the twoing criterion
  //
  return max_a;
}
