// file: $DT/class/tree/v1.0/dt_gain_1.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: continuous_gainratio_split_cc
//
// arguments: 
//  float_4& max_split (input): the maximum mask
//  Attribute_node* attr_node (input): the attribute node we are
//                                     interested in
//  Tree_node* node (input): the tree node we are currently trying to split
//
// return: float_4 indicating the gain ratio
//
// this method performs gain ratio spliting rule for continuous values
// and returns the value associated with the gain ratio
//
float_4 Decision_tree::continuous_gainratio_split_cc(float_4& max_split_a,
						   Attribute_node* attr_node_a,
						   Tree_node* node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_gainratio_split_cc",
                     (char_1*)"unable to get data nodes from in tree node");
  }
  
  // get the number of data nodes in the linked list
  //
  int_4 num_data_nodes = 0;
  if (node_a->get_num_data_nodes_cc(num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_gainratio_split_cc",
                     (char_1*)"unable to get number of data nodes in tree node");
  }
    
  if (num_data_nodes <= 0) {
    error_handler_cc((char_1*)"continous_gainratio_split",
                     (char_1*)"num_data_nodes <= 0");
  }
 
  // get the attribute value for each data node
  //
  float_4* attr_values = new float_4 [num_data_nodes];
  char_1* attr_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  if (attr_node_a->get_label_cc(attr_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_gainratio_split",
                     (char_1*)"unable to get attribute label");
  }
  
  for (int_4 i=0; i<num_data_nodes; i++) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();
    if (data_node == (Data_node*)NULL) {
      error_handler_cc((char_1*)"continous_gainratio_split",
                       (char_1*)"NULL data node");
    }

    if (data_node->get_value_cc(attr_values[i], attr_label) != ISIP_TRUE) {
      error_handler_cc((char_1*)"continous_gainratio_split",
                       (char_1*)"unable to get attribute value");
    }
    data_list = data_list->get_next_cc();
  }
  
  // sort the attribute values
  //
  int_4* index = new int_4 [num_data_nodes];
  if (qsort_cc(index, attr_values, num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_gainratio_split_cc",
                     (char_1*)"unable to sort data");
  }
  
  // loop through all possible splits for this attribute
  //
  int_4 num_classes;
  float_4 total_info=0;
  float_4 info=0;
  int_4 left, right;
  float_4 gain, gain_ratio, split_info;
  int_4 num_data;
  float_4 max_a=0;
  
  class_node_d->get_num_values_cc(num_classes);
  
  total_info = calculate_node_info_cc(num_classes, node_a);

  for (int_4 i=0; i<num_data_nodes-1; i++) {
    float_4 split = (attr_values[index[i]] + attr_values[index[i+1]])/2;
    
    info = calculate_info_cc(split, num_classes, attr_node_a, node_a,
			     num_data, left, right);
    
    gain = total_info - info;
    split_info = calculate_split_info_cc(left, right, num_data);
    if (split_info == 0) {
      if (total_info == info) {
	gain_ratio =0;
      }
      else {
	gain_ratio = max_a;
      }
    }
    else {
      gain_ratio = gain / split_info;
    }
    
    // keep the maximum split
    //
    if ( gain_ratio > max_a ) {
      max_a = gain_ratio;
      max_split_a = split;
    }
  }
    
  // delete scratch space
  //
  delete [] attr_values;
  delete [] index;
  if (attr_label != (char_1*)NULL) {
    delete [] attr_label;
  }
  
  // return max value of the twoing criterion
  //
  return max_a;
}
