// file: $DT/class/tree/v1.0/dt_info_0.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: calculate_info_cc
//
// arguments: 
//  int_4  num_classes (input): the number of classes
//  Tree_node* node (input): the tree node we are interested in calculating
//                           the information
//
// return: float_4 indicating the information on this tree node
//
// this method calculates the information conveys in a tree node
//
float_4 Decision_tree::calculate_node_info_cc(int_4 num_classes,
					 Tree_node* node_a) {

  int_4 i;
  float_4 info = 0;
  int_4* counts = new int_4[num_classes];
  int_4 num_data_nodes;
  char_1** class_values = (char_1**)NULL;
  char_1* data_class = (char_1*)NULL;
  Linked_list* ll;
  Data_node* data_node;
  
  // get data from tree node
  //
  node_a->get_num_data_nodes_cc(num_data_nodes);
  node_a->get_data_nodes_cc(ll);

  // get data from class node
  //
  class_values = new char_1*[num_classes];
  for (i=0;i<num_classes;i++) {
    class_values[i] = new char_1[ISIP_MAX_STRING_LENGTH];
  }
  class_node_d->get_values_cc(class_values);

  // initialize counts
  //
  for (i=0;i<num_classes;i++) {
    counts[i]=0;
  }

  data_class = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // add in the counts of each of the classes in the data nodes
  //
  for (i=0;i<num_data_nodes;i++) {
    data_node = (Data_node*) (ll->get_node_cc());
    data_node->get_item_cc(data_class, (char_1*)"index");

    for (int_4 j=0;j<num_classes;j++) {
      if (strcmp ((char*)data_class, (char*)class_values[j]) == 0) {
	counts[j]++;
      }
    }
    ll = ll->get_next_cc();
  }

  // calculate the information in the node
  //
  for (i=0;i<num_classes;i++) {
    info = info + entropy_cc(counts[i], num_data_nodes);
  }

  // clean up
  //
  delete [] counts;
  if (data_class != (char_1*)NULL) {
    delete [] data_class;
  }
  
  if ((num_classes > 0) && (class_values != (char_1**)NULL)) {
    for (i=0;i<num_classes;i++) {
      if (class_values[i] != (char_1*)NULL) {
	delete [] class_values[i];
	class_values[i] = (char_1*)NULL;
      }
    }
    delete [] class_values;
  }
  
  // return the information
  //
  return info;
  
}
