// file: $DT/class/tree/v1.0/dt_info_3.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: calculate_info_cc
//
// arguments: 
//  int_4  num_classes (input): the number of classes
//  Tree_node* node (input): the tree node we are interested in calculating
//                           the information
//
// return: float_4 indicating the information on this tree node
//
// this method calculates the information conveys in a tree node
//
float_4 Decision_tree::calculate_info_cc(float_4 split_a,
					 int_4 num_classes,
					 Attribute_node* attr_node_a,
					 Tree_node* node_a,
					 int_4& num_data_nodes,
					 int_4& left, int_4& right) {

  int_4 i;
  float_4 info = 0;
  int_4* count_left = new int_4[num_classes];
  int_4* count_right = new int_4[num_classes];
  char_1* attr_label_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1** attr_values_a;
  char_1** class_values;
  float_4 data_value;
  char_1* data_class = (char_1*)NULL;
  int_4 num_values_a;
  Linked_list* ll;
  Data_node* data_node;
  
  // get data from tree node
  //
  node_a->get_num_data_nodes_cc(num_data_nodes);
  node_a->get_data_nodes_cc(ll);
  
  // get data from attribute node
  //
  attr_node_a->get_label_cc(attr_label_a);
  attr_node_a->get_num_values_cc( num_values_a );
  attr_values_a = new char_1*[num_values_a];
  for (i=0; i<num_values_a; i++ ) {
    attr_values_a[i] = new char_1[ISIP_MAX_STRING_LENGTH];
  }
  attr_node_a->get_values_cc(attr_values_a);
  
  // get data from class node
  //
  class_values = new char_1*[num_classes];
  for (i=0; i<num_classes; i++ ) {
    class_values[i] = new char_1[ISIP_MAX_STRING_LENGTH];
  }
  class_node_d->get_values_cc(class_values);
  
  // initialize counts
  //
  for (i=0;i<num_classes;i++) {
    count_left[i]=0;
    count_right[i]=0;
  }
  
  data_class = new char_1[ISIP_MAX_STRING_LENGTH];
  left = 0;
  right = 0;
  
  // add in the counts of each of the classes in the data nodes
  //
  for (i=0;i<num_data_nodes;i++) {
    data_node = (Data_node*) (ll->get_node_cc());
    data_node->get_value_cc(data_value, attr_label_a);
    data_node->get_item_cc(data_class, (char_1*)"index");

    if (data_value < split_a) {
      left++;
      int_4 k=0;
      while (strcmp ((char*)data_class, (char*)class_values[k]) != 0) {
	k++;
      }
      count_left[k]++;
    }
    else {
      right++;
      int_4 k=0;
      while (strcmp ((char*)data_class, (char*)class_values[k]) != 0) {
	k++;
      }
      count_right[k]++;
    }
    ll=ll->get_next_cc();
  }
  
  float_4 left_info = 0;
  float_4 right_info = 0;
  
  // calculate the information 
  //
  for (i=0;i<num_classes;i++) {
    left_info = left_info + entropy_cc(count_left[i], left);
    right_info = right_info + entropy_cc(count_right[i], right);
  }

  info = (float_4) left / (float_4) num_data_nodes * left_info + 
    (float_4) right / (float_4) num_data_nodes * right_info;
  
  // clean up
  //
  delete [] count_left;
  delete [] count_right;
  if (data_class != (char_1*)NULL) {
    delete [] data_class;
  }
  for (i=0; i<num_values_a; i++ )
    delete [] attr_values_a[i];
  
  for (i=0; i<num_classes; i++ )
    delete [] class_values[i];
 
  delete [] attr_label_a;
  delete [] attr_values_a;
  delete [] class_values;
  
  // return the information
  //
  return info;
  
}
