// file: $DT/class/tree/v1.0/dt_io_0.cc
//

// isip include files
//
#include "tree_node.h"
#include "tree.h"
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: read_cc
//
// arguments: Tree_node* parent_a  parent to node getting inserted
//
// return: logical_1 flag to indicate success
//
// this method inserts a node into the tree at a specified location
//
logical_1 Decision_tree::read_tree_cc( char_1* fname, Tree* tree_a ) {

  char_1* input_str_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* count_str_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* prob_str_a = new char_1[ISIP_MAX_STRING_LENGTH];
  logical_1 added_a;
  Tree_node* new_node_a;
  Tree_node* parent_a;
  int_4 level_a = 0;
  int_4 prev_level_a = 0;
  FILE* fp_a;
  
  fp_a = fopen((char*)fname, "r");

  if (fp_a == (FILE*)NULL) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"no file opened");
    return ISIP_FALSE;
  }

  // insert the root of the tree
  //
  Tree_node* prev_node_a = new Tree_node();
  prev_level_a = 0;
  tree_a->insert_cc(prev_node_a, (Tree_node*)NULL);

  
  // read one line at a time from input file
  //
  while ( fgets((char*)input_str_a, ISIP_MAX_STRING_LENGTH, fp_a) != NULL ) {
    
    // if not blank line
    //
    if ((input_str_a[0] != (char_1)'\n')) {
      
      // get counts and probabilities
      //
      fgets((char*)count_str_a, ISIP_MAX_STRING_LENGTH, fp_a);
      fgets((char*)prob_str_a, ISIP_MAX_STRING_LENGTH, fp_a);

      if ( (count_str_a[0]!=(char_1)'c') || (prob_str_a[0]!=(char_1)'p') ) {
	error_handler_cc((char_1*)"read_cc",
		     (char_1*)"counts or probabilities incorrectly formatted");
      }
	
      new_node_a = new Tree_node();
      
      added_a = parse_tree_cc(level_a, new_node_a, (char_1*)input_str_a,
			      (char_1*)count_str_a, (char_1*)prob_str_a);
      
      // node to be added is on the same level as the previously added node
      //
      if ( level_a == prev_level_a ) {
	parent_a = prev_node_a->get_parent_cc();
	tree_a->insert_cc(new_node_a, parent_a);
      }
      
      // node to be added is child of previous node
      //
      else if ( level_a > prev_level_a ) {
	tree_a->insert_cc(new_node_a, prev_node_a);
      }
      
      // node to be added is higher in tree than previous node
      //
      else if ( level_a < prev_level_a ) {
	
	parent_a = prev_node_a->get_parent_cc();
	
	// get to correct tree level
	//
	for ( int_4 i=0; i< (prev_level_a-level_a); i++ ){ 
	  parent_a = parent_a->get_parent_cc();
	}
	
	tree_a->insert_cc(new_node_a, parent_a);
      }
      else {
	delete [] input_str_a;
	delete [] count_str_a;
	delete [] prob_str_a;
	fclose(fp_a);
	return ISIP_FALSE;
      }
      prev_level_a = level_a;
      prev_node_a = new_node_a;
    }
  }

  if ( input_str_a != (char_1*)NULL )
    delete [] input_str_a;
  if ( count_str_a != (char_1*)NULL )
    delete [] count_str_a;
  if ( prob_str_a != (char_1*)NULL )
    delete [] prob_str_a;
  
  fclose(fp_a);
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
