// file: $DT/class/tree/v1.0/dt_parse_3.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"
#include "decision_tree.h"
#include "param_file.h"
#include <integral_constants.h>
#include <libgen.h>
#include <strings.h>
#include <ctype.h>

// method: parse_cc
//
// arguments: Tree_node* parent_a  parent to node getting inserted
//
// return: logical_1 flag to indicate success
//
// this method reads in a node from the output file and inserts the info
// into a node in the tree
//
logical_1 Decision_tree::parse_tree_cc( int_4 &level_a, Tree_node* node_a,
					char_1* input_a, char_1* count_a,
					char_1* prob_a) {

  float_4 min_a;
  float_4 max_a;
  char_1 char_a;
  char_1* attr_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* oper1_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* oper2_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* values_a = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* substr_a;
  logical_1 range_a;
  int_4 oper;
  int_4*  count_list = new int_4[ISIP_MAX_STRING_LENGTH];
  float_4* prob_list = new float_4[ISIP_MAX_STRING_LENGTH];
  char_1** classes_a;
  int_4 split_type_a;
  int_4 num_classes_a;
  
  class_node_d->get_num_values_cc(num_classes_a);
  classes_a = new char_1*[num_classes_a];
  for ( int_4 i=0; i<num_classes_a; i++ )
    classes_a[i]=new char_1[ISIP_MAX_STRING_LENGTH];
  class_node_d->get_values_cc(classes_a);
  
  // read in tag
  //
  sscanf((char*)input_a, "%*[ <]%c%ld%*[>]", &char_a, &level_a);

  // determine if this is a binary or trinary question
  //
  if ( char_a == 'B' ) {
    range_a = ISIP_FALSE;
  }
  else {
    range_a = ISIP_TRUE;
  }

  // remove the tag from the string
  //
  substr_a = (char_1*)strchr((char*)input_a, '>');
  substr_a++;

  // scan in the rest of the line
  //
  if ( range_a ) {
    sscanf((char*)substr_a, "%*[ ]%f%*[^>^<^=]%[><=]%[^>^<^=]%[<>=]%f",
	   &min_a, (char*)oper1_a, (char*)attr_a, (char*)oper2_a, &max_a);
    split_type_a=TREE_NODE_CONTINUOUS;
  }
  else {
    sscanf((char*)substr_a, "%*[ ]%[^>^<^=]%[><=]", (char*)attr_a,
	   (char*)oper1_a);
  }

  // get representation for operator
  //
  char_1* ptr = oper1_a;
  while ( isspace(*ptr) )
    ptr++;
  switch (*ptr) { 
  case '>':
    ptr++;
    if (*ptr == '=') {
      oper = TREE_NODE_GTE;
    }
    else {
      oper = TREE_NODE_GT;
    }
    break;
  case '<':
    ptr++;
    if (*ptr == '=') {
      oper = TREE_NODE_LTE;
    }
    else {
      oper = TREE_NODE_LT;
    }
    break;
  case '=':
    oper = TREE_NODE_EQ;
    break;
  default:
    error_handler_cc((char_1*)"parse_tree_cc",
		     (char_1*)"incorrect operator format");
  }

  // node type is discrete
  //
  if ( (oper==TREE_NODE_EQ) ){
    sscanf((char*)substr_a, "%*[ ]%[^>^<^=]%[><=]%[^\n]", (char*)attr_a,
	   (char*)oper1_a, (char*)values_a);
    split_type_a=TREE_NODE_DISCRETE;
  }
  // node type is continuous
  //
  if ( (!range_a) && (oper!=TREE_NODE_EQ) ) {
    sscanf((char*)substr_a, "%*[ ]%[^>^<^=]%[><=]%f", (char*)attr_a,
	   (char*)oper1_a, &min_a);
    max_a = min_a;
    split_type_a=TREE_NODE_CONTINUOUS;
  }

  // format the attribute name
  //
  char_1* attr_name = new char_1[ISIP_MAX_STRING_LENGTH];
  ptr = attr_a;
  while ( *ptr != '\0' ) {
    if ( !isspace(*ptr) ){
      strncat((char*)attr_name, (char*)ptr, 1);
    }
    ptr++;
  }

  // read in the counts
  //
  if ( ((char*)ptr=(strstr((char*)count_a,(char*)TREE_NODE_NO_COUNT )))==(char*)NULL){
    char_1* temp = new char_1[ISIP_MAX_STRING_LENGTH];
    char_1* junk;
    strcat((char*)temp, (char*)count_a);
    junk=(char_1*)strtok((char*)temp,":");
    for ( int_4 i=0; i<num_classes_a; i++ ){
      junk=(char_1*)strtok(NULL,":");
      junk=(char_1*)strtok(NULL," ");
      convert_string_cc( count_list[i], junk );
    }
    delete [] temp;
    node_a->set_count_cc( count_list, num_classes_a );
  }

  // read in the probabilities
  //
  if ( ((char*)ptr=(strstr((char*)prob_a,(char*)TREE_NODE_NO_PROBS )))==(char*)NULL){
    char_1* temp = new char_1[ISIP_MAX_STRING_LENGTH];
    char_1* junk;
    strcat((char*)temp, (char*)prob_a);
    junk=(char_1*)strtok((char*)temp,":");
    for ( int_4 i=0; i<num_classes_a; i++ ){
      junk=(char_1*)strtok(NULL,":");
      junk=(char_1*)strtok(NULL," ");
      convert_string_cc( prob_list[i], junk );
    }
    delete [] temp;
    node_a->set_probability_cc( prob_list, num_classes_a );
  }

  // if discrete, format the attribute labels
  //
  if ( split_type_a == TREE_NODE_DISCRETE ) {
    static char_1* attr_values_a[ISIP_MAX_STRING_LENGTH/2];
    memset(attr_values_a,0,(ISIP_MAX_STRING_LENGTH/2)*sizeof(char_1*));

    static Param_file p_file;
    int_4 num_values_a;

    p_file.tokenize_cc( attr_values_a, num_values_a, values_a, (char_1)',');

    for ( int_4 i=0; i<num_values_a; i++ ) {
      char_1* buff = attr_values_a[i];
      buff++;
      strcpy((char*)attr_values_a[i],(char*)buff);
    }
    
    node_a->set_node_cc( attr_values_a , num_values_a , attr_name);
  }

  if ( split_type_a == TREE_NODE_CONTINUOUS ) {
    node_a->set_node_cc( min_a, max_a, oper, attr_name);
  }  
  
  // set the new node to be added with the correct info
  //
  node_a->set_level_cc( level_a );
  node_a->set_classes_cc( num_classes_a, classes_a );

  for ( int_4 i=0; i<num_classes_a; i++ ) {
    delete [] classes_a[i];
  }

  delete [] classes_a;
  delete [] oper1_a;
  delete [] oper2_a;
  delete [] attr_name;
  delete [] attr_a;
  delete [] count_list;
  delete [] prob_list;
  delete [] values_a;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
