// file: $DT/class/tree/v1.0/dt_split_1.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"    
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: best_split_cc
//
// arguments: int_4 dt_split_a  constant defining splitting rule used
//            Tree_node* node_a node for which the split is being found
//
// return: logical_1 indicating success
//
// this method determines the best split( discrete or continuous ) for a node
// and then applies this split
//
logical_1 Decision_tree::best_split_cc(int_4 dt_split_a, Tree_node* node_a,
				       Tree_node* left_child,
				       Tree_node* right_child) {

  // place to store the maximum splits found for either discrete or cont
  //
  float_4 max_cont_split=(float)NULL;
  float_4 max_split_a=0;             
  int_4* max_mask_a=(int_4*)NULL;
  float_4 max_a=-10000;
  int_4 max_type_a;
  Attribute_node* max_attr;

  // discrete splits
  //
  int_4* mask_a=(int_4*)NULL;
  int_4 num_values_a;

  // continuous splits
  //
  float_4 cont_split;

  float_4 calc_a;
  char_1* current_attr_a;
  int_4 type_a;
  Attribute_node* attr_node_a;
  Linked_list* ll_a;
  char_1** class_values_a;
  
  ll_a= attribute_list_d;
    
  // search every attribute
  //
  for ( int_4 i=0; i<num_of_attributes_d; i++ ) {

    attr_node_a=(Attribute_node*)ll_a->get_node_cc();

    // determine if attribute is discrete or continuous
    //
    attr_node_a->get_type_cc(type_a);
    
    // calculate the best split for each attribute
    //
    if ( type_a == ATTR_TYPE_DISCRETE ) {

      attr_node_a->get_num_values_cc(num_values_a);
      
      mask_a = new int_4[num_values_a];
      switch ( dt_split_a ) {
      case DT_SPLIT_ENT: 
	calc_a=discrete_entropy_split_cc( mask_a, num_values_a, attr_node_a,
					  node_a);
	break;
      case DT_SPLIT_BAYES:
	calc_a=discrete_bayes_split_cc( mask_a, attr_node_a, node_a);
	break;
      case DT_SPLIT_GAINRATIO:
	calc_a=discrete_gainratio_split_cc( mask_a, num_values_a, attr_node_a, node_a);
	break;
      }
    }
    else {
      switch ( dt_split_a ) {
      case DT_SPLIT_ENT: 
	calc_a=continuous_entropy_split_cc( cont_split, attr_node_a, node_a );
	break;
      case DT_SPLIT_BAYES:
	calc_a=continuous_bayes_split_cc( cont_split, attr_node_a, node_a );
	break;
      case DT_SPLIT_GAINRATIO:
	calc_a=continuous_gainratio_split_cc(cont_split, attr_node_a, node_a);
	break;
      }
    }

    // save the maximum split info
    //
    if ( calc_a > max_a ) {
      max_a = calc_a;
      max_attr=attr_node_a;
      max_type_a = type_a;
      if ( type_a == ATTR_TYPE_DISCRETE ) {
	if ( max_mask_a != (int_4*)NULL )
	  delete [] max_mask_a;
	max_mask_a = new int_4[num_values_a];
	for ( int_4 j=0; j< num_values_a; j++ ) max_mask_a[j]=mask_a[j];
      }
      else {
	max_cont_split = cont_split;
      }
    }

    if ( type_a == ATTR_TYPE_DISCRETE )
      delete [] mask_a;

    // get the next attribute_node
    //
    ll_a=ll_a->get_next_cc();
  }

  logical_1 purity;
  // apply the split to left and right hand children nodes
  //
  if ( max_type_a == ATTR_TYPE_DISCRETE ) {
    purity = purity_check_cc(max_mask_a, node_a, max_attr);
    if (purity == ISIP_TRUE) {
      apply_split_cc(max_mask_a, max_attr, node_a, left_child, right_child);
    }
  }
  else {
    purity = purity_check_cc(max_cont_split, node_a, max_attr);
    if (purity == ISIP_TRUE) {
      apply_split_cc(max_cont_split,max_attr,node_a,left_child,right_child);
    }
  }

  // add number of classes to the tree nodes
  //
  if (purity == ISIP_TRUE) {
    int_4 num_classes_a;
    class_node_d->get_num_values_cc(num_classes_a);
    class_values_a = new char_1*[num_classes_a];
    for ( int_4 i=0; i< num_classes_a; i++ )
      class_values_a[i]=new char_1[ISIP_MAX_STRING_LENGTH];
    class_node_d->get_values_cc( class_values_a );
    left_child->set_classes_cc( num_classes_a, class_values_a);
    right_child->set_classes_cc( num_classes_a, class_values_a);
    left_child->set_count_cc();
    right_child->set_count_cc();
  
    // insert the children into the node
    //
    tree_d->insert_cc( left_child, node_a );
    tree_d->insert_cc( right_child, node_a );
    for ( int_4 i=0; i<num_classes_a; i++ )
      delete [] class_values_a[i];
    delete [] class_values_a;
  }
  
  // exit gracefully
  //
  return purity;
}
