// file: $DT/class/tree/v1.0/dt_split_2.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"    
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: apply_split_cc
//
// arguments: int_4* mask_a  mask for the discrete values
//            attr_node_a  attribute where the split occurs
//            parent_a  node to be split
//            left_a, right_a   children to which the split is applied
//
// return: logical_1 indicating success
//
// this method applies the already determined best split to the node
// this split is for the discrete split
// in the mask:  the zero's go left, the one's go right
//
logical_1 Decision_tree::apply_split_cc(int_4* mask_a,
					Attribute_node* attr_node_a,
					Tree_node* parent_a, Tree_node* left_a,
					Tree_node* right_a) {

  Linked_list* ll_data;
  Data_node* data_a;
  int_4 num_values_a;
  int_4 split_direction;
  char_1* attr_name_a= new char_1[ISIP_MAX_STRING_LENGTH];

  // get length of the mask
  //
  attr_node_a->get_num_values_cc(num_values_a);

  // get the linked list of data at the parent node
  //
  parent_a->get_data_nodes_cc(ll_data);

  // classsify each data node
  //
  while ( ll_data != (Linked_list*)NULL ) {

    data_a=(Data_node*)ll_data->get_node_cc();

    // determine if the data goes in the left or right node
    //
    split_direction=classify_data_node_cc(mask_a, attr_node_a, data_a);

    // add pointer to data node to correct tree node child
    //
    if ( split_direction == DT_SPLIT_LEFT ) {
      left_a->add_data_node_cc(data_a);
    }
    else {
      right_a->add_data_node_cc(data_a);
    }

    // get the next data node pointer
    //
    ll_data=ll_data->get_next_cc();
  }

  // add info to the nodes
  //
  attr_node_a->get_label_cc( attr_name_a );
  left_a->set_node_cc( mask_a, attr_node_a, DT_SPLIT_LEFT, TREE_NODE_EQ,
		       attr_name_a);
  right_a->set_node_cc(mask_a, attr_node_a, DT_SPLIT_RIGHT, TREE_NODE_EQ,
		       attr_name_a);

  delete [] attr_name_a;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


