// file: $DT/class/tree/v1.0/dt_split_8.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "decision_tree.h"
#include "decision_tree_constants.h"

// systme include files
//
#include <integral_constants.h>

// method: purity_check_cc
//
// arguments:
//  float_4 split_a (input): splitting value for continuous data
//  Tree_node* node_a (input): the tree node location
//
// return: ISIP_TRUE if the node is splittable
//         ISIP_FALSE if the node is not splittable
//
// this method determines if the tree is trying to split into subtrees
// with one leaf has 0 cases in the training data
//
logical_1 Decision_tree::purity_check_cc (float_4 split_a, Tree_node* node_a,
					  Attribute_node* attr_node_a) {

  Linked_list* ll_data;
  Data_node* data_a;
  int_4 split_direction;
  int_4 left=0, right=0;
  logical_1 purity = ISIP_TRUE;

  // get the linked list of the data at the tree node
  //
  node_a->get_data_nodes_cc(ll_data);

  // classify each data node
  //
  while ( ll_data != (Linked_list*)NULL ) {
 
    data_a = (Data_node*)ll_data->get_node_cc();
 
    // determine if the data goes in the left or right node
    //
    split_direction = classify_data_node_cc(split_a, attr_node_a, data_a);

    // count the splits
    //
    if (split_direction == DT_SPLIT_LEFT) {
      left++;
    }
    else if (split_direction == DT_SPLIT_RIGHT) {
      right++;
    }

    ll_data=ll_data->get_next_cc();
  }

  // check for purity
  //
  if ((left == 0) || (right == 0)) {
    purity = ISIP_FALSE;
  }

  // return value
  //
  return purity;

}

