// file: $DT/class/v1.0/tree/tree.h
//
 
// make sure the definitions are made only once
//
#ifndef __ISIP_TREE
#define __ISIP_TREE

// isip include files
//
#ifndef __ISIP_INTEGRAL
#include <integral.h>
#endif

// forward class definitions
//
#include <tree_node.h>
#include <decision_tree_constants.h>

// Tree: a class that defines a general tree structure
//
class Tree {
  
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:
  
  // declare a variable for debugging
  //
  int_4 debug_level_d;

  // pointer to the root of the tree
  //
  Tree_node* root_d;

  // pointer to current node of the tree
  //
  Tree_node* current_d;

  // number of terminal nodes in tree
  //
  int_4 terminal_d;

  
  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // required methods
  //
  char_1* name_cc();
  volatile void error_handler_cc(char_1* mname_a, char_1* msg_a);
  logical_1 debug_cc(FILE* fp_a, char_1* message_a);
  logical_1 set_debug_cc(int_4 debug_level_a);

  // destructor/constructor
  //
  ~Tree();
  Tree();

  // overloaded operator for copying tree into a new tree
  //
  Tree& operator=( const Tree& old_tree );

  // tree operations
  //
  logical_1 insert_cc(Tree_node* new_node_a, Tree_node* parent_a);
  logical_1 delete_cc(Tree_node* node_a);
  Tree_node* get_root_cc();
  Tree_node* get_current_node_cc();
  int_4 number_terminal_nodes_cc(int_4 terminal_a=0,
				 Tree_node* current_a=(Tree_node*)NULL);

  // write tree to file
  //
  logical_1 output_tree_cc(FILE* fp_a, Tree_node* current_a,
			   int_4 display_level_a = DT_DISPLAY_BASIC);
  logical_1 output_tree_cc(FILE* fp_a,
			   int_4 display_level_a = DT_DISPLAY_BASIC);
  logical_1 output_detail_tree_cc(FILE* fp_a, Tree_node* current_a);


  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

};
#endif



