// file: $DT/class/tree_node/v1.0/tree_node.h
//
 
// make sure the definitions are made only once
//
#ifndef __ISIP_TREE_NODE
#define __ISIP_TREE_NODE

// isip include files
//
#ifndef __ISIP_INTEGRAL
#include <integral.h>
#endif

// forward class definitions
//
#include <linked_list.h>
#include <data_node.h>
#include <attribute_node.h>
// tree_node is a container for the data included at each node of a tree
//
class Tree_node {
  
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:
  
  // declare a variable for debugging
  //
  int_4 debug_level_d;   
  
  // node hierarchy
  //
  Tree_node* parent_d;
  Linked_list* children_d;

  // indicates whether this node contains discrete or continuous split
  // TREE_NODE_DISCRETE or TREE_NODE_CONTINUOUS
  //
  int_4 split_type_d;

  // data contained at each node
  //
  char_1* label_d;                    // attribute name
  int_4 operator_d;                   // operator defined in constants file
  char_1* question_d;                 // question asked at the node
  char_1* output_d;                   // question asked+level+formatting
  float_4* probability_d;             // prob associated with this node
  int_4* count_d;                     // count associated with this node
  int_4 level_d;                      // level of the tree this node resides
  int_4 num_class_d;                  // number of classes for this data set
  char_1** classes_d;                 // strings containing class labels

  // continuous case specific variables
  //
  float_4 min_d;                      // minimum value for the range
  float_4 max_d;                      // maximum value for the range

  // discrete case specific variables
  //
  char_1** discrete_values_d;         // strings containing discrete labels
  int_4 num_discrete_values_d;        // number of discrete values
  
  // a linked list of pointers to the data used at this node
  //
  Linked_list* data_d;
  int_4 num_data_nodes_d;
  
  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // required methods
  //
  char_1* name_cc();
  volatile void error_handler_cc(char_1* mname_a, char_1* msg_a);
  logical_1 debug_cc(FILE* fp_a, char_1* message_a);
  logical_1 set_debug_cc(int_4 debug_level_a);

  // destructor/constructor
  //
  ~Tree_node();
  Tree_node();
  Tree_node(float_4 min_a, float_4 max_a, int_4 operator_d,
	    char_1* attr_label_a);
  Tree_node(char_1** discrete_values_a, int_4 num_discrete_a, int_4 operator_a,
	    char_1* attr_label_a);
  Tree_node(const Tree_node& old_node);

  // node operations
  //
  int_4 number_of_children_cc();
  logical_1 add_child_cc(Tree_node* child_a);
  logical_1 ask_question_cc(float_4 test_a);
  logical_1 ask_question_cc(char_1* test_a);
  logical_1 is_terminal_cc();

  // set methods
  //
  logical_1 set_node_cc(float_4 min_a, float_4 max_a, int_4 operator_d,
			char_1* attr_label_a);
  logical_1 set_node_cc(int_4* mask_a, Attribute_node* node_a, int_4 split_a,
			int_4 operator_a, char_1* attr_label_a);
  logical_1 set_node_cc(char_1** discrete_values_a, int_4 num_a,
			char_1* attr_label_a );
  logical_1 set_parent_cc(Tree_node* parent_a);
  logical_1 set_level_cc( int_4 level_a );
  logical_1 set_count_cc();
  logical_1 set_count_cc( int_4* count_list, int_4 num_classes );
  logical_1 set_probability_cc( float_4* probability_a, int_4 size_a );
  logical_1 set_classes_cc(int_4 num_class_a, char_1** classes_a );
  
  // get methods
  //
  Linked_list* get_children_cc();
  char_1* get_attribute_cc();
  Tree_node*  get_parent_cc();
  int_4 get_level_cc();
  char_1* get_contents_cc(int_4 display_level_a);
  float_4* get_probability_cc();
  int_4* get_count_cc();
  char_1* get_detailed_cc();
  int_4 get_split_type_cc();
  
  // methods to control data used at this node for splitting and pruning
  //
  logical_1 add_data_node_cc( Data_node* data_ptr );
  logical_1 get_num_data_nodes_cc( int_4& num_nodes_a );
  logical_1 get_data_nodes_cc( Linked_list*& ll_a );
  
  // helps determine if test case belongs to this node or not
  //
  char_1* assign_class_cc();

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

  // private get method
  //
  char_1* get_question_cc();

  // i/o methods
  //
  logical_1 print_probs_cc(char_1* prob_str_a);
  logical_1 print_counts_cc(char_1* count_str_a);

  
};

#endif



