# File : display.tcl
#

# This file sets up the display for the N-best pronunciations
# Boltzmann machine network program
#

#import globals
#
global p
global num_n_best
global pron_context
global name_spell
global pron_bmp

# withdraw the existing top level so that a new one can be set
#
wm withdraw .

# if required the top level may be destroyed
#
catch {destroy $p}

# now set up the new top level
#
toplevel $p

# use packing as no size is decided for $p
#
wm title $p $pron_title

# if the window is deleted the program should quit
#
wm protocol $p WM_DELETE_WINDOW {
    quit_proc
}

# #####################################################
# now create the frames that assist in packing the demo
# #####################################################

# the two major frames
#
frame $p.inputs_frame
frame $p.outputs_frame

# subframes inside the inputs frame
#
frame $p.in_lt_frame 
frame $p.in_rt_frame

#subframe inside the input left frame
#
frame $p.in_lt1_frame
frame $p.in_lt2_frame

#subframe inside the input left second frame (lt2)
#
frame $p.in_lt21_frame
frame $p.in_lt22_frame

#subframe inside the input left second frame (lt22)
#
frame $p.in_lt22_nn_frame
frame $p.in_lt22_dt_frame

# subframes inside the input right frame
#
frame $p.in_rt1_frame
frame $p.in_rt2_frame
frame $p.in_rt3_frame
frame $p.out_lt_frame
frame $p.out_rt_frame
frame $p.nbest_frame

# ###############################
# set up the inputs frame widgets
# ###############################

# set up the info, quit and run buttons
#
button $p.run -text Run -command {run_proc}
button $p.play -text Play -command {play_all_proc}
button $p.clear -text Clear -command {clear_proc}
button $p.quit -text Quit -command {quit_proc}
button $p.info -text Info -command {info_proc}

# set up the up and down n_best buttons
#
button $p.up_n_best -bitmap "@$pron_bmp/incr.xbm" -command {
    incr num_n_best 1
    if {$num_n_best > $max_n_best} {set num_n_best $max_n_best}
}
button $p.dn_n_best -bitmap "@$pron_bmp/decr.xbm" -command {
    incr num_n_best -1
    if {$num_n_best < $min_n_best} {set num_n_best $min_n_best}
}

# set up the context, algorithm, volume, n_best, and spelling labels
#
label $p.context -text "Context" -relief raised -bg grey -fg black
label $p.algorithm -text "Algorithm" -relief raised -bg grey -fg black
label $p.nn -text "Neural Net" -bg grey -fg black
label $p.dt -text "Decision Tree" -bg grey -fg black
label $p.n_best -text "N best" -relief raised -bg grey -fg black
label $p.noun_spelling -text "Spelling" -relief raised -bg grey -fg black

# set up the radiobuttons for the context variable
#
radiobutton $p.context3 -text "Context 3" -variable pron_context \
	-value 3 
radiobutton $p.context5 -text "Context 5" -variable pron_context \
	-value 5
radiobutton $p.context7 -text "Context 7" -variable pron_context \
	-value 7

# set up the radiobuttons for the algorithm variable
#
radiobutton $p.dt_bayes \
	-text "Bayes    " -variable pron_alg \
	-value bayes
radiobutton $p.dt_cart \
	-text "CART     " -variable pron_alg \
	-value cart
radiobutton $p.dt_c4 \
	-text "C4       " -variable pron_alg \
	-value c4
radiobutton $p.nn_boltzmann \
	-text "Boltzmann" -variable pron_alg \
	-value boltzmann
radiobutton $p.nn_mlp \
	-text "MLP      " -variable pron_alg \
	-value mlp

# set up the entry widgets for n_best and the input word spelling
#
entry $p.n_best_entry -width 2 -relief sunken -textvariable num_n_best
entry $p.spelling_entry -relief sunken -width 50 -textvariable name_spell
bind $p.n_best_entry <Return> {run_proc}
bind $p.spelling_entry <Return> {
    run_proc 
    play_all_proc
}

# setup the audio widgets
#
scale $p.volume -variable volume -from 0 -to 5 -digits 2 \
	-resolution 0.1 -showvalue false -orient horiz
label $p.volume_label -text "Volume" -relief raised
button $p.audio_cfg -text "Audio Config" -command audio_cfg_proc

# ################################
# set up the outputs frame widgets
# ################################

# the text widget to display outputs
#
label $p.outputs -text "N-best pronunciations" -relief raised \
	-bg grey -fg black
text $p.pron_outs -height 15 -width 40 -relief sunken -wrap none\
	-yscrollcommand "$p.scrolly set" \
	-xscrollcommand "$p.scrollx set" \
	-background $back_color -foreground $fore_color -state disabled \
	
scrollbar $p.scrolly -command "$p.pron_outs yview"
scrollbar $p.scrollx -command "$p.pron_outs xview" -orient horizontal

# the canvas widget to draw the graph of connections
#
label $p.phones -text "Phoneme network" -relief raised -bg grey -fg black
canvas $p.network -height 200 -width 400 -relief sunken \
	-yscrollcommand "$p.yscroll set" \
	-xscrollcommand "$p.xscroll set" \
	-borderwidth 2
scrollbar $p.yscroll -command "$p.network yview"
scrollbar $p.xscroll -command "$p.network xview" -orient horizontal

# ##############################
# pack the layout frames into $p
# ##############################

pack $p.inputs_frame $p.outputs_frame -fill x -side top -expand true -in $p

# pack the widgets in the inputs frame
#
pack $p.in_lt_frame -side left -in $p.inputs_frame -expand true
	
pack $p.in_rt_frame -side right -in $p.inputs_frame -fill x -expand true

pack $p.context $p.context3 $p.context5 \
	 $p.context7 -side top -ipadx 2m -pady 0.3m \
	-in $p.in_lt1_frame -fill both -expand true

pack $p.nn $p.nn_boltzmann $p.nn_mlp -expand false \
	-side top -ipadx 2m -pady 0.3m -in $p.in_lt22_nn_frame -fill both

pack $p.dt $p.dt_bayes $p.dt_cart $p.dt_c4 -expand false \
	-side top -ipadx 2m -pady 0.3m -in $p.in_lt22_dt_frame -fill both

pack $p.in_lt22_nn_frame $p.in_lt22_dt_frame -expand true \
	-side left -ipadx 2m -pady 0.3m -in $p.in_lt22_frame -fill both

pack $p.algorithm -expand true \
	-side top -ipadx 2m -pady 0.3m -in $p.in_lt21_frame -fill both

pack $p.in_lt21_frame $p.in_lt22_frame -expand true \
	-side top -ipadx 2m -pady 0.3m -in $p.in_lt2_frame -fill both

pack $p.in_lt1_frame $p.in_lt2_frame -side left -fill x \
	-expand true -padx 1m -pady 1m -in $p.in_lt_frame -anchor n

pack $p.run $p.play $p.clear $p.quit $p.info -side left -fill x \
	-padx 1m -pady 1m -expand true -in $p.in_rt1_frame

pack $p.up_n_best -side top -fill both -expand true -padx 1 -pady 1 \
	-in $p.nbest_frame

pack $p.dn_n_best -side bottom -fill both -expand true -padx 1 -pady 1 \
	-in $p.nbest_frame

pack $p.n_best $p.n_best_entry $p.nbest_frame -side left -fill x \
	-padx 1m -pady 1m -in $p.in_rt2_frame -expand true

pack $p.volume $p.audio_cfg -side right -fill x -expand true \
	-padx 1 -pady 1 -in $p.in_rt2_frame

pack $p.noun_spelling $p.spelling_entry -side left -fill x -expand true \
	-padx 1m -pady 1m -in $p.in_rt3_frame

pack $p.in_rt1_frame $p.in_rt2_frame $p.in_rt3_frame -side top -fill x \
	-expand true -padx 1m -pady 1m -in $p.in_rt_frame

# pack the widgets in the outputs frame
#
pack $p.out_lt_frame -side left -padx 3m -pady 3m -in $p.outputs_frame \
	-expand true
pack $p.out_rt_frame -side right -padx 3m -pady 3m -in $p.outputs_frame \
	-expand true

pack $p.outputs -in $p.out_lt_frame -side top -expand true -fill x
pack $p.scrolly -in $p.out_lt_frame -fill y -side right
pack $p.scrollx -in $p.out_lt_frame -side bottom -fill x
pack $p.pron_outs -in $p.out_lt_frame -side top -expand true -fill x

pack $p.phones -in $p.out_rt_frame -side top -expand true -fill x
pack $p.yscroll -in $p.out_rt_frame -side right -fill y
pack $p.xscroll -in $p.out_rt_frame -side bottom -fill x
pack $p.network -in $p.out_rt_frame -side top -expand true -fill x








