# File: draw_net_proc.tcl
#
 
# this procedure draws a network of phonemes for pronunciations
# generated using a Boltzmann machine
#

proc draw_net_proc {} {
    
    # import globals
    #
    global p
    global num_row
    global name_spell
    global net_file
    global name_length
    global nodex
    global nodey
    global n_text
    global edge1
    global edge2
    
    # length of spelling
    #
    set name_length [string length $name_spell]
    set num_row 0
    set xx 10

    # remove everything if there is already some network
    #
    catch {$p.network delete all}
    catch {unset nodex nodey n_text edge1 edge2}
    
    # open file
    #
    catch {
	set filename [open $net_file]
	while {![eof $filename]} {
	    
	    # draw the corresponding nodes
	    #
	    set yy  [expr $num_row*50]
	    set xx  0
	    
	    catch {
		for {set i 0} { $i < $name_length} {incr i} {
		    scan $line "%s" txt
		    set zz  50
		    if { ([regexp "_" $txt]) } {
			set zz 10
		    }
		    set xx [expr $xx+$zz]
		    make_node $xx $yy $i $txt $num_row
		    set line [string trimleft $line $txt]
		    set line [string trimleft $line]
		}
	    }
	    
	    # increment number of levels
	    #
	    incr num_row
	    
	    # read next line of file
	    #
	    gets $filename line
	}
	
	# close file
	#
	close $filename
    }
    # now connect edges
    #
    connect_net_proc

    # reconfigure the scroll region to encompass the whole image
    #
    set size [$p.network bbox all]
 
    set x0 [lindex $size 0]
    set y0 [lindex $size 1]
    set x1 [lindex $size 2]
    set y1 [lindex $size 3]

    set x0 [expr $x0 - 5]
    set x1 [expr $x1 + 5]
    set y0 [expr $y0 - 5]
    set y1 [expr $y1 + 5]

    set size {}
    set size [linsert $size 0 $x0 $y0 $x1 $y1]

    $p.network configure -scrollregion $size

}

# #########################################
# procedure to connect nodes in the network
# #########################################

proc connect_net_proc {} {
    
    # import globals
    #
    global p
    global num_row
    global name_length
    global n_text

    # connect the nodes
    #
    for {set k 1} {$k < $num_row} {incr k} {
	for {set j 1} {$j < $num_row} {incr j} {
	    for {set i 0} {$i < [expr $name_length-1]} {incr i} {
		
		set txt $n_text($k,$i)
		if { (![regexp "_" $txt]) && (![regexp "#" $txt]) } {
		    
		    set l [expr $i+1]
		    if {$l >= $name_length} {
			break;
		    }
		    
		    set txt $n_text($j,$l)
		    if { [regexp "#" $txt] } {
			for {set q 1} {$q < $j} {incr q} {
			    if { [regexp "_" $n_text($q,$l)] } {
				incr l
				if {$l >= $name_length} {
				    break;
				}
				set txt $n_text($j,$l)
				break;
			    }
			}
		    }
		    
		    while { [regexp "_" $txt] } {
			incr l
			if {$l >= $name_length} {
			    break;
			}
			set txt $n_text($j,$l)
		    }
		    
		    if { ($l < $name_length) && \
			    (![regexp "#" $n_text($j,$l)]) } {
			make_edge $k $i $j $l
		    }
		}
	    }
	}
    }
}


# ################################################################
# procedure to draw a node at (x, y) with tag ii and text label tt
# ################################################################

proc make_node { x y ii tt nn } {

    # declare globals
    #
    global p
    global nodex
    global nodey
    global n_text
    global edge1
    global edge2

    if { (![regexp "_" $tt]) && (![regexp "#" $tt]) } {
	
	# draw the node
	#
	set new [$p.network create oval [expr $x-16] [expr $y-16] \
		[expr $x+16] [expr $y+16] -outline black -fill white \
		-tags node]
	
	# set the label
	#
	$p.network create text [expr $x-10] [expr $y+12] -text $tt \
		-font -adobe-times-bold-i-normal--*-180-75-75-p-98-iso8859-1 \
		-anchor sw -tags node
    }

    # set the globals for reference
    #
    set nodex($nn,$ii) $x
    set nodey($nn,$ii) $y
    set n_text($nn,$ii) $tt
    set edge1($nn,$ii) {}
    set edge2($nn,$ii) {}
}


# ##############################################
# procedure to draw an edge connecting two nodes
# ##############################################

proc make_edge { nn1 ii1 nn2 ii2 } { 

    # declare globals
    #
    global p
    global nodex
    global nodey
    global n_text
    global edge1
    global edge2
    
    # make the connection
    #
    catch {
	
	set xx2 $nodex($nn2,$ii2)
	set yy2 $nodey($nn2,$ii2)
	
	if { ($nn1 == $nn2) } {
	    set xx2 [expr $xx2-12]
	}
	
	if { ($nn1 < $nn2) } {
	    set yy2 [expr $yy2-10]
	    set xx2 [expr $xx2-12]
	}

	if { ($nn1 > $nn2) } {
	    set yy2 [expr $yy2+15]
	}

	set edge [$p.network create line $nodex($nn1,$ii1) \
		$nodey($nn1,$ii1) $xx2 $yy2 \
		-arrow last -tags edgenet]

	$p.network lower $edge
	
	# update globals
	#
	lappend edge1($nn1,$ii1) $edge
	lappend edge2($nn2,$ii2) $edge
    }
}
