# File: info_proc.tcl
#

# this procedure provides some information on the Boltzmann machine 
# system for N-best generation of pronunciations
#

proc info_proc {} {

    # import globals
    #
    global p
    global lp
    global info_text
    global pron_bmp

    set newline \n

    # set the text message
    #
    set info_text "

    This application uses a three-layered Boltzmann machine network to derive the N best pronunciations for a proper noun specified in the 'Noun Spelling' entryfield. \n
    
    It uses small groups of adjacent letters as context to compute the likely phonemes that describe the pronunciation. The user can select the desired context by clicking on the appropriate radiobutton. The 'short' context considers 3 adjacent letters (1 on either side of the letter of interest) while the 'long' context uses 7 adjacent letters (3 on either side of the letter of interest). The 'short+long' context uses the knowledge from each of the individual contexts above, while the 'both' context uses knowledge derived by simultaneously applying both contexts. \n
    
    The user can also specify the number of possible pronunciations generated by specifying a value in the 'N best' entry field. This number is set to have a range from 1 to 100."
    
    # set the name for the new toplevel
    #
    set lp "$p.lp"
    
    # destroy the new toplevel if it exists
    #
    catch {destroy $lp}
 
    # create the new toplevel
    #
    toplevel $lp -class Dialog
    wm title $lp "Brief Description"
    wm iconname $lp Info
    
    # create the frames that will be used in packing
    #
    frame $lp.top_frame -relief raised -bd 1
    frame $lp.bot_frame -relief raised -bd 1

    # pack the two frames
    #
    pack $lp.top_frame -side top -fill both
    pack $lp.bot_frame -side bottom -fill both

    # create the bitmap label for info
    #
    label $lp.info_map -bitmap "@$pron_bmp/isip_logo.xbm"

    # create the message widget to display info
    #
    message $lp.info_msg -width 3.5i -justify left \
	    -text $info_text
    
    # bind the return key to quit this window
    #
    bind $lp <Return> {destroy $lp}
    
    # set up the dismiss button
    #
    button $lp.dismiss -text Dismiss -command {destroy $lp}
    
    # pack the frames
    #
    pack $lp.info_map -side left -in $lp.top_frame -fill both \
	    -padx 3m -pady 3m -expand true
    pack $lp.info_msg -side right -in $lp.top_frame -fill both \
	    -padx 3m -pady 3m -expand true
    pack $lp.dismiss -side left -in $lp.bot_frame -expand true
}
