# File: run_proc.tcl
#
 
# this procedure runs the Boltzmann machine code with the entered
# parameters
#
 
proc run_proc {} {

    # import globals
    #
    global p
    global num_n_best
    global name_spell
    global pron_context
    global pron_alg
    global pron_params
    global params_dir
    global dt_params_dir
    global in_data
    global in_file
    global tmp1_file
    global tmp2_file
    global out_file
    global net_file
    global tree_file
    global generate_pron
    global build_net
    global name_to_context
    global generate_tree
    global contextpron_to_name
    global rebuild

    # change the cursor to wait
    #
    $p config -cursor watch
    update

    # determine which algorithm
    #
    if { ([regexp "boltzmann" $pron_alg]) } { 

	# load the appropriate params for the boltzmann machine
	#
	if { ([regexp "3" $pron_context]) } { 
	    set pron_params "$params_dir/neural_network/context_3.params"
	}
	if { ([regexp "5" $pron_context]) } { 
	    set pron_params "$params_dir/neural_network/context_5.params"
	}
	if { ([regexp "7" $pron_context]) } { 
	    set pron_params "$params_dir/neural_network/context_7.params"
	}

	# write the input name into the datafile
	#
	exec echo $name_spell > $in_data
	exec echo $in_data > $in_file
    
	# run the boltzmann network on the specified string
	#
	exec $generate_pron -f test -p $pron_params -n $num_n_best \
		-i $in_file -o $out_file
    }
    if { ([regexp "mlp" $pron_alg]) } { 

	# load the appropriate params for the boltzmann machine
	#
	if { ([regexp "3" $pron_context]) } { 
	    set pron_params "$params_dir/neural_network/context_3.params"
	}
	if { ([regexp "5" $pron_context]) } { 
	    set pron_params "$params_dir/neural_network/context_5.params"
	}
	if { ([regexp "7" $pron_context]) } { 
	    set pron_params "$params_dir/neural_network/context_7.params"
	}

	# write the input name into the datafile
	#
	exec echo $name_spell > $in_data
	exec echo $in_data > $in_file
    
	# run the mlp network on the specified string
	#
	exec $generate_pron -f mlp -p $pron_params -n $num_n_best \
		-i $in_file -o $out_file
    }
    if { ([regexp "bayes" $pron_alg]) } { 

	# load the appropriate params for the decision tree
	#
	if { ([regexp "3" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_3_bayes.tree"
	}
	if { ([regexp "5" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_5_bayes.tree"
	}
	if { ([regexp "7" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_7_bayes.tree"
	}

	# run the decision tree network on the specified string
	#

	# write the input name into the datafile
	#
	exec echo $name_spell > $in_data

	# convert it to format recognized to the tree
	#
	set pos [expr $pron_context/2 + 1]

	exec $name_to_context $pron_context $pos $in_data > $in_file

	# test the converted name 
	#
	exec $analyze_dtree -param $param_file -attr $attr_file \
		$in_file $tmp1_file
    
	exec $contextpron_to_name -file $tmp1_file -context $pron_context \
		-column $pos -pron 2 > $tmp2_file

	# convert the output of the tree to word/pron pair
	#
	exec $rebuild -input $tmp2_file	-context $pron_context \
		-nbest $num_n_best -max 2 -output $out_file
    }
    if { ([regexp "cart" $pron_alg]) } { 

	# load the appropriate params for the decision tree
	#
	if { ([regexp "3" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_3_cart.tree"
	}
	if { ([regexp "5" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_5_cart.tree"
	}
	if { ([regexp "7" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_7_cart.tree"
	}

	# run the decision tree network on the specified string
	#

	# write the input name into the datafile
	#
	exec echo $name_spell > $in_data

	# convert it to format recognized to the tree
	#
	set pos [expr $pron_context/2 + 1]

	exec $name_to_context $pron_context $pos $in_data > $in_file

	# test the converted name 
	#
	exec $analyze_dtree -param $param_file -attr $attr_file \
		$in_file $tmp1_file
    
	exec $contextpron_to_name -file $tmp1_file -context $pron_context \
		-column $pos -pron 2 > $tmp2_file

	# convert the output of the tree to word/pron pair
	#
	exec $rebuild -input $tmp2_file	-context $pron_context \
		-nbest $num_n_best -max 2 -output $out_file
    }
    if { ([regexp "c4" $pron_alg]) } { 

	# load the appropriate params for the decision tree
	#
	if { ([regexp "3" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_3_c4.tree"
	}
	if { ([regexp "5" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_5_c4.tree"
	}
	if { ([regexp "7" $pron_context]) } { 
	    set pron_params "$params_dir/decision_tree/context_7_c4.tree"
	}

	# run the decision tree network on the specified string
	#

	# write the input name into the datafile
	#
	exec echo $name_spell > $in_data

	# convert it to format recognized to the tree
	#
	set pos [expr $pron_context/2 + 1]

	exec $name_to_context $pron_context $pos $in_data > $in_file

	# test the converted name 
	#
	exec $analyze_dtree -param $param_file -attr $attr_file \
		$in_file $tmp1_file
    
	exec $contextpron_to_name -file $tmp1_file -context $pron_context \
		-column $pos -pron 2 > $tmp2_file

	# convert the output of the tree to word/pron pair
	#
	exec $rebuild -input $tmp2_file	-context $pron_context \
		-nbest $num_n_best -max 2 -output $out_file
    }

    # load the output in buffer display
    # this also prepares for playing names audibly
    #
    readfile_proc
        
    # sort the output to get only the different pronunciations
    # then get only the different phonemes in a network
    #
    catch {exec rm $net_file}
    exec $build_net $out_file $net_file $num_n_best
    
    # draw the network using this file
    #
    draw_net_proc
    
    # restore the cursor
    #
    $p config -cursor top_left_arrow
    update
}






