// file bm_comp_0.cc
//
// method to compute the expected (target) output of the machine in training
//

// isip include files
//
#include "boltz_machine.h"
#include <boltz_characters.h>

//-----------------------------------------------------------------------------
// method: tgt_out_cc
//
// arguments:
//     char_1* phone_a: (input) the phoneme string
//     float_4* outs_a: (output) the output bits vector
//
// return: a logical_1 to indicate success
//
// this method computes the expected values of the output bits of the 
// boltzmann machine from the input pronunciation string during training
// 
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: tgt_out_cc (char_1* phone_a, float_4* outs_a) {

  // size of the bit vector in bytes
  //
  static int_4 vector_size_l = (int_4)BITS_PER_OUT_PHONE * sizeof (float_4);

  // now assign the output vector depending on the phoneme
  //
  for (int_4 i = 0; i < NUM_PHONEMES; i++) {
    if (strcmp ((char*)phone_a, (char*)output_phoneme_d[i]) == 0) {
      memcpy (outs_a, phone_vector_d[i], vector_size_l);

      // exit gracefully
      //
      return (ISIP_TRUE);
    }
  }
  
  // symbol does not belong to phoneme set
  //
  fprintf (stderr, "Symbol %s is not a valid phoneme.\n", phone_a);
  
  // exit with error
  //
  exit (ISIP_ERROR);
}
