// file bm_comp_2.cc
//
// method to compute the output bits of the boltzmann machine 
// for one input letter
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: compute_outputs_cc ()
//
// arguments: none
//
// return: a logical_1 to indicate success
//
// this method computes the output bits for one input letter
// 
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: compute_outputs_cc () {

  //
  // first compute the output of the first hidden layer neurons
  //

  // temperary variables
  //
  float_4* inputs_hid_l = NULL;
  int_4 i = 0;
  
  // work on the appropriate half of the machine according to context -- short
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)) {
    
    // get the inputs to the first hidden layer
    //
    inputs_hid_l = new float_4[num_short_inputs_d];
    input_short_d->output_cc (inputs_hid_l);
    
    // compute the output for each neuron
    //
    for (i = 0; i < num_hidden_short_d[0]; i++) {
      
      // calculate the energy gap
      //
      hidden_short_d[0][i].set_energy_cc (wt_in_hid_short_d, inputs_hid_l, 
					  num_short_inputs_d + 1, i);

      // calculate the output value
      //
      hidden_short_d[0][i].set_output_cc (operation_d);
    }

    // free memory
    //
    delete inputs_hid_l;
  }

  // work on the appropriate half of the machine according to context -- long
  //
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)) {
    
    // get the inputs to the hidden layer
    //
    inputs_hid_l = new float_4[num_long_inputs_d];
    input_long_d->output_cc (inputs_hid_l);
    
    // compute the output for each neuron
    //
    for (i = 0; i < num_hidden_long_d[0]; i++) {
      
      // calculate the energy gap
      //
      hidden_long_d[0][i].set_energy_cc (wt_in_hid_long_d, inputs_hid_l, 
					 num_long_inputs_d + 1, i);
      
      // calculate the output value
      //
      hidden_long_d[0][i].set_output_cc (operation_d);
    }
    
    // free memory
    //
    delete inputs_hid_l;
  }
  
  //---------------------------------------------------------------------------
  //
  // next compute the output of the other hidden layer neurons
  //
  //---------------------------------------------------------------------------
  
  // temporary variables
  //
  float_4* inputs_out_l = NULL;
  float_4** weights_l = NULL;
  int_4 num_weights_l = 0;
  int_4 k = 0;

  // for short context
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)) {
    
    for (k = 0; k < num_layers_short_d - 1; k++) {
      
      // get the inputs to the next hidden layer
      //
      inputs_out_l = new float_4[num_hidden_short_d[k]];
      for (i = 0; i < num_hidden_short_d[k]; i++) {
	hidden_short_d[k][i].output_cc (inputs_out_l[i]);
      }
      
      // set the weights
      //
      num_weights_l = num_hidden_short_d[k] + 1;
      weights_l = wt_hid_out_short_d[k];
      
      // compute the output for each neuron
      //
      for (i = 0; i < num_hidden_short_d[k+1]; i++) {
	
	// calculate the energy gap
	//
	hidden_short_d[k+1][i].set_energy_cc (weights_l, inputs_out_l, 
					      num_weights_l, i);
	
	// calculate the output value
	//
	hidden_short_d[k+1][i].set_output_cc (operation_d);
      }
    
      // free memory
      //
      delete inputs_out_l;
      inputs_out_l = (float_4*)NULL;
    }
  }

  // for long context
  //
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)) {
    
    for (k = 0; k < num_layers_long_d - 1; k++) {
      
      // get the inputs to the next hidden layer
      //
      inputs_out_l = new float_4[num_hidden_long_d[k]];
      for (i = 0; i < num_hidden_long_d[k]; i++) {
	hidden_long_d[k][i].output_cc (inputs_out_l[i]);
      }
      
      // set the weights
      //
      num_weights_l = num_hidden_long_d[k] + 1;
      weights_l = wt_hid_out_long_d[k];
      
      // compute the output for each neuron
      //
      for (i = 0; i < num_hidden_long_d[k+1]; i++) {
	
	// calculate the energy gap
	//
	hidden_long_d[k+1][i].set_energy_cc (weights_l, inputs_out_l, 
					      num_weights_l, i);
	
	// calculate the output value
	//
	hidden_long_d[k+1][i].set_output_cc (operation_d);
      }
    
      // free memory
      //
      delete inputs_out_l;
      inputs_out_l = (float_4*)NULL;
    }
  }

  //---------------------------------------------------------------------------
  //
  // finally compute the output of the output layer neurons
  //
  //---------------------------------------------------------------------------
  
  // temporary variables
  //
  inputs_out_l = (float_4*)NULL;
  weights_l = (float_4**)NULL;
  num_weights_l = 0;
  int_4 ind_l = 0;
  
  // for any single context
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_LONG)) {
    
    // for short context
    //
    if (context_d == BOLTZ_SHORT) {

      // set index to last hidden layer
      //
      ind_l = num_layers_short_d - 1;
      
      // get the inputs to the output layer
      //
      inputs_out_l = new float_4[num_hidden_short_d[ind_l]];
      for (i = 0; i < num_hidden_short_d[ind_l]; i++) {
	hidden_short_d[ind_l][i].output_cc (inputs_out_l[i]);
      }
      
      // set the weights
      //
      num_weights_l = num_hidden_short_d[ind_l] + 1;
      weights_l = wt_hid_out_short_d[ind_l];
    }
    
    // for long context
    //
    else {
      
      // set index to last hidden layer
      //
      ind_l = num_layers_long_d - 1;
      
      // get the inputs to the output layer
      //
      inputs_out_l = new float_4[num_hidden_long_d[ind_l]];
      for (i = 0; i < num_hidden_long_d[ind_l]; i++) {
	hidden_long_d[ind_l][i].output_cc (inputs_out_l[i]);
      }
      
      // set the weights
      //
      num_weights_l = num_hidden_long_d[ind_l] + 1;
      weights_l = wt_hid_out_long_d[ind_l];
    }
    
    // compute the output for each neuron
    //
    for (i = 0; i < num_output_neurons_d; i++) {
      
      // calculate the energy gap
      //
      output_neuron_d[i].set_energy_cc (weights_l, inputs_out_l, 
					num_weights_l, i);
      
      // calculate the output value
      //
      output_neuron_d[i].set_output_cc (operation_d);
    }
    
    // free memory
    //
    delete inputs_out_l;
    inputs_out_l = (float_4*)NULL;
  }
  
  // if both contexts are in use
  //
  else if (context_d == BOLTZ_BOTH) {

    int_4 sh_l = num_layers_short_d - 1;
    int_4 ln_l = num_layers_long_d - 1;
    
    // get the inputs to the output layer of neurons
    //
    float_4* inputs_out_short_l = new float_4[num_hidden_short_d[sh_l]];
    for (i = 0; i < num_hidden_short_d[sh_l]; i++) {
      hidden_short_d[sh_l][i].output_cc (inputs_out_short_l[i]);
    }
    float_4* inputs_out_long_l = new float_4[num_hidden_long_d[ln_l]];
    for (i = 0; i < num_hidden_long_d[ln_l]; i++) {
      hidden_long_d[ln_l][i].output_cc (inputs_out_long_l[i]);
    }
    
    // compute the output for each neuron
    //
    for (i = 0; i < num_output_neurons_d; i++) {
      
      // calculate the energy gap
      //
      output_neuron_d[i].set_energy_both_cc (wt_hid_out_short_d[sh_l], 
					     inputs_out_short_l, 
					     num_hidden_short_d[sh_l] + 1,
					     wt_hid_out_long_d[ln_l],
					     inputs_out_long_l, 
					     num_hidden_long_d[ln_l] + 1,
					     i);
      
      // calculate the output value
      //
      output_neuron_d[i].set_output_cc (operation_d);
    }
    
    // free memory
    //
    delete inputs_out_short_l;
    delete inputs_out_long_l;
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
