// file bm_comp_3.cc
//
// method to compute the output error of each neuron in the boltzmann machine 
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: compute_errors_cc
//
// arguments:
//     float_4* out_a: (input) the expected output of the machine
//
// return: a logical_1 to indicate success
//
// this method computes the error terms during training for each neuron and
// accordingly sets the error vectors
// 
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: compute_errors_cc (float_4* out_a) {

  // temporary variables
  //
  int_4 i = 0;
  int_4 j = 0;
  int_4 ind_l = 0;
  int_4 pre_l = 0;
  float_4 out_bit_l = 0;
  float_4 bk_prop_err_l = 0;
  float_4** weight_l = NULL;

  // first calculate the errors in the output layer
  //
  for (i = 0; i < num_output_neurons_d; i++) {
    output_neuron_d[i].output_cc (out_bit_l);
    error_out_d[i] = out_bit_l * (1.0 - out_bit_l) * (out_a[i] - out_bit_l);
  }
  
  // now backpropagate these errors to compute error in hidden layers
  //
  // for the short context hidden layers
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)) {
    
    // assign index variables
    //
    ind_l = num_layers_short_d - 1;
    weight_l = wt_hid_out_short_d[ind_l];
    
    // for each neuron in the hidden layer connected to the output layer
    //
    for (i = 0; i < num_hidden_short_d[ind_l]; i++) {
      
      // calculate backpropagation term
      //
      bk_prop_err_l = 0;
      for (j = 0; j < num_output_neurons_d; j++) {
	bk_prop_err_l += error_out_d[j] * weight_l[i + 1][j];
      }
      
      // calculate scaled error term for the neuron
      //
      hidden_short_d[ind_l][i].output_cc (out_bit_l);
      error_hid_short_d[ind_l][i] = out_bit_l * (1.0 - out_bit_l) *
	bk_prop_err_l;
    }
    
    // for the subsequent hidden layers proceeding backwards
    //
    while (ind_l > 0) {
      
      // index of current and previous hidden layers
      //
      pre_l = ind_l;
      ind_l --;
      
      // weights coming out of this layer
      //
      weight_l = wt_hid_out_short_d[ind_l];
      
      // for each neuron in the hidden layer connected to the previous layer
      //
      for (i = 0; i < num_hidden_short_d[ind_l]; i++) {
	
	// calculate backpropagation term
	//
	bk_prop_err_l = 0;
	for (j = 0; j < num_hidden_short_d[pre_l]; j++) {
	  bk_prop_err_l += error_hid_short_d[pre_l][j] * weight_l[i + 1][j];
	}
	
	// calculate scaled error term for the neuron
	//
	hidden_short_d[ind_l][i].output_cc (out_bit_l);
	error_hid_short_d[ind_l][i] = out_bit_l * (1.0 - out_bit_l) *
	  bk_prop_err_l;
      }
    }
  }

  // for the long context hidden layers
  //
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)) {
    
    // assign index variables
    //
    ind_l = num_layers_long_d - 1;
    weight_l = wt_hid_out_long_d[ind_l];
    
    // for each neuron in the hidden layer connected to the output layer
    //
    for (i = 0; i < num_hidden_long_d[ind_l]; i++) {
      
      // calculate backpropagation term
      //
      bk_prop_err_l = 0;
      for (j = 0; j < num_output_neurons_d; j++) {
	bk_prop_err_l += error_out_d[j] * weight_l[i + 1][j];
      }
      
      // calculate scaled error term for the neuron
      //
      hidden_long_d[ind_l][i].output_cc (out_bit_l);
      error_hid_long_d[ind_l][i] = out_bit_l * (1.0 - out_bit_l) *
	bk_prop_err_l;
    }
    
    // for the subsequent hidden layers proceeding backwards
    //
    while (ind_l > 0) {
      
      // index of current and previous hidden layers
      //
      pre_l = ind_l;
      ind_l --;
      
      // weights coming out of this layer
      //
      weight_l = wt_hid_out_long_d[ind_l];
      
      // for each neuron in the hidden layer connected to the previous layer
      //
      for (i = 0; i < num_hidden_long_d[ind_l]; i++) {
	
	// calculate backpropagation term
	//
	bk_prop_err_l = 0;
	for (j = 0; j < num_hidden_long_d[pre_l]; j++) {
	  bk_prop_err_l += error_hid_long_d[pre_l][j] * weight_l[i + 1][j];
	}
	
	// calculate scaled error term for the neuron
	//
	hidden_long_d[ind_l][i].output_cc (out_bit_l);
	error_hid_long_d[ind_l][i] = out_bit_l * (1.0 - out_bit_l) *
	  bk_prop_err_l;
      }
    }
  }

  // exit gracefully
  //
  return (ISIP_TRUE);
}
