// file bm_comp_4.cc
//
// method to compute the update values for the connection weights
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: compute_updates_cc
//
// arguments:
//     float_4* out_a: (input) the expected output of the machine
//
// return: a logical_1 to indicate success
//
// this method computes the update terms for the connection weights
// 
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: compute_updates_cc (float_4* out_a) {

  // calculate the error terms
  //
  compute_errors_cc (out_a);

  // now calculate the update terms for each connection weight
  //

  // dummy variables
  //
  int_4 i = 0;
  int_4 k = 0;
  int_4 ind_l = 0;
  int_4 row_l = 0;
  int_4 col_l = 0;
  float_4* inputs_l = NULL;
  float_4** up_weight_l = NULL;
  
  // for short context
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)) {
    
    // for the hidden layer connected to the output layer
    //
    ind_l = num_layers_short_d - 1;
    up_weight_l = up_hid_out_short_d[ind_l];
    row_l = num_hidden_short_d[ind_l];
    col_l = num_output_neurons_d;
        
    inputs_l = new float_4[row_l];
    for (i = 0; i < row_l; i++) {
      hidden_short_d[ind_l][i].output_cc (inputs_l[i]);
    }
      
    // compute updates for each weight in this matrix
    //
    update_weight_cc (up_weight_l, error_out_d, inputs_l, row_l + 1, col_l);

    // free memory
    //
    delete inputs_l;
    inputs_l = NULL;
    
    // for subsequent hidden layers
    //
    for (k = ind_l; k > 0; k--) {

      up_weight_l = up_hid_out_short_d[k-1];
      row_l = num_hidden_short_d[k-1];
      col_l = num_hidden_short_d[k];

      inputs_l = new float_4[row_l];
      for (i = 0; i < row_l; i++) {
	hidden_short_d[k-1][i].output_cc (inputs_l[i]);
      }
        
      // compute updates for each weight in this matrix
      //
      update_weight_cc (up_weight_l, error_hid_short_d[k], inputs_l,
			row_l + 1, col_l);

      // free memory
      //
      delete inputs_l;
      inputs_l = NULL;
    }
    
    // for weights connecting the input layer to the first hidden layer
    //
    up_weight_l = up_in_hid_short_d;
    row_l = num_short_inputs_d;
    col_l = num_hidden_short_d[0];
    
    inputs_l = new float_4[row_l];
    input_short_d->output_cc (inputs_l);

    // compute updates for each weight in this matrix
    //
    update_weight_cc (up_weight_l, error_hid_short_d[0], inputs_l,
		      row_l + 1, col_l);
    
    // free memory
    //
    delete inputs_l;
    inputs_l = NULL;
  }
	
  // for long context
  //
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)) {
    
    // for the hidden layer connected to the output layer
    //
    ind_l = num_layers_long_d - 1;
    up_weight_l = up_hid_out_long_d[ind_l];
    row_l = num_hidden_long_d[ind_l];
    col_l = num_output_neurons_d;
    
    inputs_l = new float_4[row_l];
    for (i = 0; i < row_l; i++) {
      hidden_long_d[ind_l][i].output_cc (inputs_l[i]);
    }
    
    // compute updates for each weight in this matrix
    //
    update_weight_cc (up_weight_l, error_out_d, inputs_l, row_l + 1, col_l);
    
    // free memory
    //
    delete inputs_l;
    inputs_l = NULL;
    
    // for subsequent hidden layers
    //
    for (k = ind_l; k > 0; k--) {
      
      up_weight_l = up_hid_out_long_d[k-1];
      row_l = num_hidden_long_d[k-1];
      col_l = num_hidden_long_d[k];

      inputs_l = new float_4[row_l];
      for (i = 0; i < row_l; i++) {
	hidden_long_d[k-1][i].output_cc (inputs_l[i]);
      }
      
      // compute updates for each weight in this matrix
      //
      update_weight_cc (up_weight_l, error_hid_long_d[k], inputs_l,
			row_l + 1, col_l);
      
      // free memory
      //
      delete inputs_l;
      inputs_l = NULL;
    }
    
    // for weights connecting the input layer to the first hidden layer
    //
    up_weight_l = up_in_hid_long_d;
    row_l = num_long_inputs_d;
    col_l = num_hidden_long_d[0];
    
    inputs_l = new float_4[row_l];
    input_long_d->output_cc (inputs_l);

    // compute updates for each weight in this matrix
    //
    update_weight_cc (up_weight_l, error_hid_long_d[0], inputs_l,
		      row_l + 1, col_l);
    
    // free memory
    //
    delete inputs_l;
    inputs_l = NULL;
  }
	
  // exit gracefully
  //
  return (ISIP_TRUE);
}
