// file bm_comp_5.cc
//
// method to calculate the update values
//

// isip include files
//
#include "boltz_machine.h"
#include <boltz_constants.h>

//-----------------------------------------------------------------------------
// method: update_weight_cc
//
// arguments:
//     float_4** up_weight_a: (input) update matrix
//     float_4* error_a: (input) the error terms to propagate back
//     float_4* inputs_a: (input) the inputs to the neuron layer
//     int_4 row_a: (input) number of rows in update matrix
//     int_4 col_a: (input) number of columns in update matrix
//
// return: a logical_1 to indicate success
//
// this method computes the update terms for the connection weights in the
// given matrix
// 
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: update_weight_cc (float_4** up_weight_a,
					     float_4* error_a,
					     float_4* inputs_a,
					     int_4 row_a, int_4 col_a) {

  // dummy index variables
  //
  int_4 i = 0;
  int_4 j = 0;
  float_4 scale_l = 0;
  
  // compute the learning rate scale factor
  //
  for (j = 0; j < col_a; j++) {
    
    // the scale factor
    //
    scale_l = learning_rate_d * error_a[j];

    // increase the scale factor if error is large
    //
    if (scale_update_d == ISIP_TRUE) {
      if (fabs (error_a[j]) > (float_4)MAX_BIT_ERROR) {
	scale_l *= (float_4)UPDATE_SCALE_FACTOR;
      }
    }
    
    // update the first element of each column
    //
    up_weight_a[0][j] = momentum_d * up_weight_a[0][j] + scale_l;
  
    // compute the update terms for weights connecting the two layers
    //
    for (i = 1; i < row_a; i++) {
      up_weight_a[i][j] = momentum_d * up_weight_a[i][j] +
	scale_l * inputs_a[i - 1];
    }
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
