// file bm_cstr_1.cc
//
// constructor
//

// isip include files
//
#include "boltz_machine.h"
#include <boltz_characters.h>

//-----------------------------------------------------------------------------
// method: constructor
//
// arguments: none
//
// return: none
//
// this is the default constructor for the boltzman machine class
//-----------------------------------------------------------------------------

Boltz_machine :: Boltz_machine () {

  // initialize miscellaneous parameters
  //
  n_best_d = (int_4)DEFAULT_NBEST;
  num_run_attempts_d = (int_4)DEFAULT_NUM_ATTEMPTS;
  num_input_bits_d = (int_4)BITS_PER_IN_CHAR;
  num_output_bits_d = (int_4)BITS_PER_OUT_PHONE;
  max_rand_limit_d = (float_4)DEFAULT_RAND_LIMIT;
  operation_d = (logical_1)BOLTZ_TEST;
  scale_update_d = (logical_1)ISIP_TRUE;
  seed_d = (int_4)BOLTZ_NONE;
  context_d = (int_4)BOLTZ_NONE;

  // assign character string memory
  //
  context_type_d = new char_1[MAX_WORD_SIZE];
  seed_type_d = new char_1[MAX_WORD_SIZE];
  
  // context sizes
  //
  short_context_length_d = (int_4)SHORT_CONTEXT_SIZE;
  long_context_length_d = (int_4)LONG_CONTEXT_SIZE;
  
  // initialize number of connections
  //
  num_layers_short_d = 0;
  num_layers_long_d = 0;
  num_short_inputs_d = short_context_length_d * num_input_bits_d;
  num_long_inputs_d = long_context_length_d * num_input_bits_d;
  num_hidden_short_d = (int_4*)NULL;
  num_hidden_long_d = (int_4*)NULL;
  num_output_neurons_d = num_output_bits_d;
  
  // initialize training-related parameters
  //
  num_max_epochs_d = (int_4)DEFAULT_MAX_EPOCHS;
  max_error_d = (float_4)DEFAULT_MAX_ERROR;
  init_temp_d = (float_4)DEFAULT_INIT_TEMPERATURE;
  min_temp_d = (float_4)DEFAULT_MIN_TEMPERATURE;
  final_momentum_d = (float_4)DEFAULT_FINAL_MOMENTUM;
  exponent_d = (float_4)DEFAULT_EXPONENT;
  learning_rate_d = (float_4)DEFAULT_LEARNING_RATE;
  train_status_d = (logical_1)BOLTZ_UNTRAINED;
  num_epochs_d = 0;
  error_d = 0;
  temperature_d = init_temp_d;
  momentum_d = 0;
  
  // assign initial null values to pointers
  //
  input_short_d = (Input_buffer*)NULL;
  input_long_d  = (Input_buffer*)NULL;
  hidden_short_d = (Boltz_neuron**)NULL;
  hidden_long_d = (Boltz_neuron**)NULL;
  output_neuron_d = (Boltz_neuron*)NULL;
  
  wt_in_hid_short_d = (float_4**)NULL;
  wt_hid_out_short_d = (float_4***)NULL;
  wt_in_hid_long_d = (float_4**)NULL;
  wt_hid_out_long_d = (float_4***)NULL;
  up_in_hid_short_d = (float_4**)NULL;
  up_hid_out_short_d = (float_4***)NULL;
  up_in_hid_long_d = (float_4**)NULL;
  up_hid_out_long_d = (float_4***)NULL;
  
  error_hid_short_d = (float_4**)NULL;
  error_hid_long_d = (float_4**)NULL;
  error_out_d = (float_4*)NULL;
  
  // exit gracefully
  //
}
