// file bm_file_2.cc
//
// method to write the trained model weights to file
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: write_network_cc
//
// arguments:
//     char_1* outfile_a: (input) name of the file containing model filenames
//
// return: a logical_1 to indicate success
//
// this method reads the list of files from the input file and according
// to context writes the connection weights to files
//
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: write_network_cc (char_1* outfile_a) {

  // dummy index variables
  //
  int_4 k = 0;
  
  // open the input list of files
  //
  FILE* model_file_l = open_file_cc (outfile_a, (char_1*)"r");

  // string to read filenames
  //
  char_1* wtfile_l = new char_1[MAX_STRING_SIZE];

  // for short context weights
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)) {

    // write the filenames for in-hid and hid-out connections
    //
    fscanf (model_file_l, "%s", wtfile_l);
    print_weights_cc (num_short_inputs_d + 1, num_hidden_short_d[0], 
		      wt_in_hid_short_d, wtfile_l);

    for (k = 0; k < num_layers_short_d; k++) {
      fscanf (model_file_l, "%s", wtfile_l);
      print_weights_cc (num_hidden_short_d[k] + 1, num_hidden_short_d[k+1],
			wt_hid_out_short_d[k], wtfile_l);
    }
  }
  
  // for long context weights
  //
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)) {
    
    // write the filenames for in-hid and hid-out connections
    //
    fscanf (model_file_l, "%s", wtfile_l);
    print_weights_cc (num_long_inputs_d + 1, num_hidden_long_d[0],
		      wt_in_hid_long_d, wtfile_l);

    for (k = 0; k < num_layers_long_d; k++) {
      fscanf (model_file_l, "%s", wtfile_l);
      print_weights_cc (num_hidden_long_d[k] + 1, num_hidden_long_d[k+1],
			wt_hid_out_long_d[k], wtfile_l);
    }
  }
  
  // free strings and close files
  //
  fclose (model_file_l);
  delete [] wtfile_l;
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
