// file bm_file_3.cc
//
// method to read the connecting weights for one layer from file
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: init_weights_cc
//
// arguments:
//     int_4 rows_a: (input) number of rows of matrix
//     int_4 cols_a: (input) number of columns of matrix
//     float_4** weight_a: (input) the matrix to be loaded
//     char_1* file_a: (input) name of the file containing model filename
//
// return: a logical_1 to indicate success
//
// this method reads the connecting weights for a particular layer after
// verifying the dimensions
//
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: init_weights_cc (int_4 rows_a, int_4 cols_a, 
					    float_4** weight_a, 
					    char_1* file_a) {

  // dummy index variables
  //
  int_4 i = 0;
  int_4 j = 0;

  // the initialization is random if no file is specified
  //
  if (file_a == NULL) {
    for (i = 0; i < rows_a; i++) {
      for (j = 0; j < cols_a; j++) {
	weight_a[i][j] = boltz_random_cc (-max_rand_limit_d, 
					  max_rand_limit_d);
      }
    }
  }
  
  // otherwise load it from file
  //
  else {
    
    // open the input file
    //
    FILE* wt_file_l = open_file_cc (file_a, (char_1*)"r");
    
    // read the matrix size and verify
    //
    int_4 temp_rows_l = 0;
    int_4 temp_cols_l = 0;
    fscanf (wt_file_l, "%ld%ld", &temp_rows_l, &temp_cols_l);
    
    if ((temp_rows_l != rows_a) || (temp_cols_l != cols_a)) {
      fprintf (stderr, "Dimensions in %s do not match matrix specs.\n",
	       file_a);
      exit (ISIP_ERROR);
    }

    // read the matrix values
    //
    for (i = 0; i < rows_a; i++) {
      for (j = 0; j < cols_a; j++) {
	fscanf (wt_file_l, "%f", &weight_a[i][j]);
      }
    }
    
    // free strings and close files
    //
    fclose (wt_file_l);
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
