// file bm_load_1.cc
//
// method to load the name pronunciation from the input file
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: read_pron_cc
//
// arguments:
//     FILE* file_a: (input) the file containing name data
//     int_4 length_a: (input) the length of the name spelling
//     char_1** pron _a: (output) the pronunciation to be read
//
// return: a logical_1 to indicate success
//
// this method loads the input name pronunciation from data file
//
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: read_pron_cc (FILE* file_a, 
					 int_4 length_a, char_1** pron_a) {
  
  // read in the pronunciation string from the file
  //
  char_1* input_pron_l = new char_1[MAX_WORD_SIZE];
  fscanf (file_a, "%[\b\t]", input_pron_l);
  fscanf (file_a, "%[^\n]", input_pron_l);

  // dummy index variables
  //
  int_4 k = 0;
  int_4 i = 0;

  // now break it into phoneme components and store them
  //
  char_1* phone_l = (char_1*) strtok ((char*)input_pron_l, " ");
  strcpy ((char*)pron_a[k], (char*)phone_l);
  
  while ((char*)phone_l = strtok (NULL, " ")) {
    k ++;
    strcpy ((char*)pron_a[k], (char*)phone_l);
  }
  
  // read junk till next spelling name
  //
  fscanf (file_a, "%[\n]", input_pron_l);
  
  // free memory
  //
  delete [] input_pron_l;
  
  // check end-of-word
  //
  k++;
  if (k != length_a) {
    fprintf (stderr, "Number of phonemes not equal to number of symbols.\n");
    exit (ISIP_ERROR);
  }
  
  // clear the remaining pronunciation vector
  //
  for (i = k; i < MAX_WORD_SIZE; i++) {
    strcpy ((char*)pron_a[i], "");
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
