// file bm_run_1.cc
//
// method to run the boltzmann machine
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: test_cc
//
// arguments:
//      char_1* outfile_a: (output) the file to write pronunciations
//      char_1* wtfile_a: (input) the files that store the trained weights
//      char_1* infile_a: (input) the files containing test data
//
// return: a logical_1 to indicate success
//
// this method runs the boltzmann machine network using data from a list
// of test files and writes the pronunciations to an output file
//
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: test_cc (char_1* outfile_a, 
				    char_1* wtfile_a, char_1* infile_a) {

  // load the weights into the machine
  //
  read_network_cc (wtfile_a);

  // set the system temperature
  //
  up_temperature_cc ();

  // open the test data file
  //
  FILE* test_file_l = open_file_cc (infile_a, (char_1*)"r");
  
  // open output file
  //
  FILE* out_file_l = open_file_cc (outfile_a, (char_1*)"w");
  
  // variable to store the likelihood data about a particular pronunciation
  //
  float_4 pron_lhood_l = 0;
  float_4 lscore_l = 0;
  
  // variable to keep track of letters in input spelling
  //
  int_4 symbol_count_l = 0;
  int_4 name_length_l = 0;
  
  // variables to keep track of various pronunciations
  //
  int_4 num_prons_l = 0;
  Output_pron* op_prons_l = (Output_pron*)NULL;
  char_1* pron_string_l = new char_1[MAX_WORD_SIZE * (MAX_PHONE_SIZE + 1)];
  char_1* phone_l = new char_1[MAX_PHONE_SIZE];
  
  // read the name of the test data file-list file
  //
  char_1* datafile_l = new char_1[MAX_STRING_SIZE];
  char_1* name_spell_l = new char_1[MAX_WORD_SIZE];
  char_1* prev_spell_l = new char_1[MAX_WORD_SIZE];

  while (fscanf (test_file_l, "%s", datafile_l) != EOF) {
    
    // open the test data file
    //
    FILE* data_file_l = open_file_cc (datafile_l, (char_1*)"r");
    
    // read the input string to the machine
    //
    while (fscanf (data_file_l, "%[^\n]", name_spell_l) != EOF) {
      
      // extract the name part from the string read
      //
      strtok ((char*)name_spell_l, "\t");
      
      // proceed with generation of pronunciations only if
      // this is a new name i.e. is not repeated from the previous one
      //
      if (strcmp ((char*)prev_spell_l, (char*)name_spell_l) != 0) {
	
	// set this to be the new name to check next time
	//
	strcpy ((char*)prev_spell_l, (char*)name_spell_l);
	
	// length of input name
	//
	name_length_l = strlen ((char*)name_spell_l);
	
	// initialize the storage array for all unique pronunciations
	//
	op_prons_l = new Output_pron (name_spell_l, n_best_d);
	
	// loop the data through numerous runs of the machine
	//
	for (int_4 nr = 0; nr < num_run_attempts_d; nr++) {
	  
	  // load the input shift register
	  //
	  load_spell_cc (name_spell_l);
	  
	  // clear string to store the pronunciation
	  //
	  memset (pron_string_l, 0, MAX_WORD_SIZE * (MAX_PHONE_SIZE + 1));

	  // for each letter in the shift register run the
	  // test, keep shifting the letters one at a time
	  // reset score before starting
	  //
	  symbol_count_l = 0;
	  pron_lhood_l = 0;
	  while (symbol_count_l < name_length_l) {
	    
	    // compute the outputs of the layers and get score
	    //
	    compute_outputs_cc ();
	    get_likelihood_cc (lscore_l);
	    pron_lhood_l += lscore_l;
	    
	    // print the corresponding phoneme to output
	    //
	    memset (phone_l, 0, MAX_PHONE_SIZE);
	    get_phoneme_cc (phone_l);
	    strcat ((char*)pron_string_l, (char*)phone_l);
	    strcat ((char*)pron_string_l, " ");
	    
	    // shift the name in the register
	    //
	    symbol_count_l ++;
	    shift_word_cc ();
	  }
	  
	  // add this pronunciation to the storage
	  //
	  op_prons_l->add_pron_cc (pron_string_l, pron_lhood_l);
	  
	  // see if we have enough pronunciations
	  //
	  op_prons_l->uniq_prons_cc (num_prons_l);
	  if (num_prons_l == n_best_d) {
	    break;
	  }
	}
	
	// write strings to output
	//
	op_prons_l->print_prons_cc (out_file_l);
	
	// free memory
	//
	delete op_prons_l;
      }
      
      // prepare to read next name
      //
      fscanf (data_file_l, "%[\n]", name_spell_l);
    }

    // close file and free memory
    //
    fclose (data_file_l);
  }
  
  // free strings and close files
  //
  fclose (out_file_l);
  fclose (test_file_l);
  delete [] name_spell_l;
  delete [] prev_spell_l;
  delete [] datafile_l;
  delete [] pron_string_l;
  delete [] phone_l;

  // exit gracefully
  //
  return (ISIP_TRUE);
}
