// file: bn_set_0.cc
//
// method to set the output of the neuron
//

// isip include files
//
#include "boltz_neuron.h"

//-----------------------------------------------------------------------------
// method: set_output_cc
//
// arguments: 
//     logical_1 function_a: (input) the operation flag -- training or testing
//
// return: a logical_1 to indicate success status
//
// this method sets the output value of a neuron at a particular temperature
//
//-----------------------------------------------------------------------------

logical_1 Boltz_neuron :: set_output_cc (logical_1 function_a) {

  // compute the output probability of firing
  //
  float_4 boltz_coeff_l = energy_gap_d * inv_temp_d;

  // the folowing thresholds are obtained using the formula
  //                   1
  //        p(x) = -----------
  //                      E/T
  //                 1 + e
  // which simplifies to
  //
  //        E/T = ln (1 - p(x)) - ln (p(x))
  //
  // The limits of +- 4.0 are chosen arbitrarily. The probability values
  // are taken as the centers of the corresponding class intervals
  //
  if (boltz_coeff_l >= 4.0)
    {probability_d = 0.0;}
  else if (boltz_coeff_l >= 2.1972)
    {probability_d = 0.05;}
  else if (boltz_coeff_l >= 1.3863)
    {probability_d = 0.15;}
  else if (boltz_coeff_l >= 0.8473)
    {probability_d = 0.25;}
  else if (boltz_coeff_l >= 0.4055)
    {probability_d = 0.35;}
  else if (boltz_coeff_l >= 0.0)
    {probability_d = 0.45;}
  else if (boltz_coeff_l >= -0.4055)
    {probability_d = 0.55;}
  else if (boltz_coeff_l >= -0.8473)
    {probability_d = 0.65;}
  else if (boltz_coeff_l >= -1.3863)
    {probability_d = 0.75;}
  else if (boltz_coeff_l >= -2.1972)
    {probability_d = 0.85;}
  else if (boltz_coeff_l >= -4.0)
    {probability_d = 0.95;}
  else
    {probability_d = 1.0;}

  // in training this is the same as output
  //
  if (function_a == BOLTZ_TRAIN) {
    output_d = probability_d;
  }

  // compute output according to probability if testing
  //
  else if (function_a == BOLTZ_TEST) {

    // set output according to probability
    //
    float_4 chances_l = boltz_random_cc (0.0, 1.0);
    if (chances_l > probability_d) {
      probability_d = 1.0 - chances_l;
      output_d = 0.0;
    }
    else {
      probability_d = chances_l;
      output_d = 1.0;
    }
  }

  else if (function_a == MLP_TEST) {

    // set most probable output
    //
    output_d = rint(probability_d);
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
