// file: bn_set_1.cc
//
// method to set the energy gap of the neuron when only one context is in play
//

// isip include files
//
#include "boltz_neuron.h"

//-----------------------------------------------------------------------------
// method: set_energy_cc
//
// arguments: 
//     float_4** weights_a: (input) the weights connecting to this layer
//     float_4* inputs_a: (input) the inputs connecting to this neuron
//     int_4 length_a: (input) the number of inputs connected
//     int_4 col_a: (input) index to the particular neuron
//
// return: a logical_1 to indicate success status
//
// this method sets the energy gap value of a neuron for a single context
//
//-----------------------------------------------------------------------------

logical_1 Boltz_neuron :: set_energy_cc (float_4** weights_a, 
					 float_4* inputs_a, 
					 int_4 length_a, int_4 col_a) {

  // dummy index variables
  //
  int_4 i = 0;
  
  // initialize with the threshold value weight contribution
  //
  energy_gap_d = weights_a[0][col_a];

  // calculate the energy gap term as the product sum of the given arrays
  //
  for (i = 1; i < length_a; i++) {

    // the very first weight is the threshold value
    // therefore start from next
    //
    energy_gap_d -= weights_a[i][col_a] * inputs_a[i - 1];
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
