// file: bn_set_2.cc
//
// method to set the energy gap of the neuron when both contexts are used
//

// isip include files
//
#include "boltz_neuron.h"

//-----------------------------------------------------------------------------
// method: set_energy_both_cc
//
// arguments: 
//     float_4** weights_short_a: (input) the short weights to this layer
//     float_4* inputs_short_a: (input) the short inputs to this neuron
//     int_4 length_short_a: (input) the number of short inputs connected
//     float_4** weights_long_a: (input) the long weights to this layer
//     float_4* inputs_long_a: (input) the long inputs to this neuron
//     int_4 length_long_a: (input) the number of long inputs connected
//     int_4 col_a: (input) index to the particular neuron
//
// return: a logical_1 to indicate success status
//
// this method sets the energy gap value of a neuron for both contexts
// (it will be used only for both-context operation for output neurons)
//
//-----------------------------------------------------------------------------

logical_1 Boltz_neuron :: set_energy_both_cc (float_4** weights_short_a, 
					      float_4* inputs_short_a, 
					      int_4 length_short_a,
					      float_4** weights_long_a, 
					      float_4* inputs_long_a, 
					      int_4 length_long_a, 
					      int_4 col_a) {
  
  // dummy index variable
  //
  int_4 i = 0;
  
  // first do the short context part
  //
  // initialize with the threshold value weight contribution
  //
  energy_gap_d = weights_short_a[0][col_a];
  
  // calculate the energy gap term as the product sum of the given arrays
  //
  for (i = 1; i < length_short_a; i++) {

    // the very first weight is the threshold value
    // therefore start from next
    //
    energy_gap_d -= weights_short_a[i][col_a] * inputs_short_a[i - 1];
  }

  // now append the long-context data
  //
  // add the threshold value weight contribution
  //
  energy_gap_d += weights_long_a[0][col_a];
  
  // calculate the energy gap term as the product sum of the given arrays
  //
  for (i = 1; i < length_long_a; i++) {

    // the very first weight is the threshold value
    // therefore start from next
    //
    energy_gap_d -= weights_long_a[i][col_a] * inputs_long_a[i - 1];
  }
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
