// file: ib_set_2.cc
//
// method to convert an input character in the buffer to corresponding bits
//

// isip include files
//
#include "input_buffer.h"
#include <boltz_characters.h>

//-----------------------------------------------------------------------------
// method: set_bits_cc
//
// arguments: 
//     float_4* bits_a: (output) bits corresponding to the character
//     char_1 symbol_a: (input) character for which bits are required
//
// return: a logical_1 value indicating success
//
// this method sets the buffer output for a particular position
//
//-----------------------------------------------------------------------------

logical_1 Input_buffer :: set_bits_cc (float_4* bits_a, char_1 symbol_a) {
  
  // the size of the bit vector
  //
  static int_4 vector_size_l = (int_4)BITS_PER_IN_CHAR * sizeof (float_4);
  int_4 index_l = -1;

  // change to lowercase
  //
  symbol_a = tolower (symbol_a);
  
  // determine the correct bit vector for this symbol
  //
  for (int_4 i = 0; i < NUM_CHARS; i++) {
    if (symbol_a == input_char_d[i]) {
      index_l = i;
      break;
    }
  }
  
  if (index_l < 0) {
    fprintf (stderr, "Symbol %c is not in valid alphabet set.\n", symbol_a);
    exit (ISIP_ERROR);
  }
  
  // set the bits according to input letter
  //
  memcpy (bits_a, char_vector_d[index_l], vector_size_l);
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}
