// file: tree_prnt_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "tree.h"

#include <integral_constants.h>
#include "tree_constants.h"

// method: read_cc
//
// arguments:
//  float_8& prob_a: (output) probability -- this has to be backed up tree
//  FILE* fp_a: (input) file pointer
//  int_4 level_a: (input) current tree level
//  char_1* prefix_a: (input) whether this should be a yes, no, or neither
//
// return:
//  a logical_1 indicating status
//
//
logical_1 Tree::read_cc(float_8& prob_a, FILE* fp_a, int_4 level_a,
			char_1* prefix_a) {

  if (fp_a == (FILE*)NULL) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"no file opened");
    return ISIP_FALSE;
  }

  if (prefix_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"prefix_a is NULL");
    return ISIP_FALSE;
  }

  prob_a = 0;
  
  char_1 line[ISIP_MAX_STRING_LENGTH];

  if ((fgets((char*)line, ISIP_MAX_STRING_LENGTH, fp_a) == (char*)NULL) ||
      strlen((char*)line) < 2) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"blank line encountered");
    return ISIP_FALSE;
  }

  int_4 space_count;
  char_1 l_or_t;
  int_4 level;

  for(space_count = 0; line[space_count] == (char_1)' '; space_count ++);
  
  if (sscanf((char*)&line[space_count], TREE_LNUM_MASK,
	      (char*)&l_or_t, &level) != 2) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"malformed line");
    return ISIP_FALSE;
  }
  if (level != level_a) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"invalid level encountered");
    return ISIP_FALSE;
  }
  else if (space_count != (level * 2)) {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"invalid space pad");
    return ISIP_FALSE;
  }

  char_1 prefix[ISIP_MAX_STRING_LENGTH];
  
  if (l_or_t == TREE_LEVEL_INDICATOR) {

    // this line is a level, not a terminal
    //

    // read in the rest of the information
    //
    if(sscanf((char*)&line[level + 3], TREE_LEVEL_IMASK,
	      (char*)prefix, &pos_d, (char*)&char_d) != 3) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"invalid level question");
      return ISIP_FALSE;
    }

    // determine if the position is valid
    //
    if (pos_d < 0) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"invalid level question position setting");
      return ISIP_FALSE;
    }

    if (char_d == TREE_LEVEL_IMASK_SPACE_CHECK) {
      char_d = (char_1)' ';
    }
    
    // determine if the character is valid
    //
    logical_1 char_flag = ISIP_FALSE;
    for (int_4 i=0; i<TREE_NUM_CHARS; i++) {
      if (char_d == chars[i]) {
	char_flag = ISIP_TRUE;
      }
    }
    if (char_flag == ISIP_FALSE) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"invalid level question char setting");
      return ISIP_FALSE;
    }

    // allocate the left node
    //
    if (left_child_d != (Tree*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"left child not NULL");
      return ISIP_FALSE;
    }
    left_child_d = new Tree(fatal_mode_d);

    if (left_child_d == (Tree*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"left child not NULL");
      return ISIP_FALSE;
    }

    // set the parent
    //
    left_child_d->parent_d = this;
    
    // read in the left node
    //
    if (left_child_d->read_cc(left_prob_d, fp_a, level_a + 1,
			      (char_1*)TREE_L_PROB_IMASK) != ISIP_TRUE) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"error training left node");
      return ISIP_FALSE;
    }

    // allocate the right node
    //
    if (right_child_d != (Tree*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"right child not NULL");
      return ISIP_FALSE;
    }
    right_child_d = new Tree(fatal_mode_d);
    if (right_child_d == (Tree*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"right child not NULL");
      return ISIP_FALSE;
    }

    // set the parent
    //
    right_child_d->parent_d = this;

    // read in the right node
    //
    if (right_child_d->read_cc(right_prob_d, fp_a, level_a + 1,
			       (char_1*)TREE_R_PROB_IMASK) != ISIP_TRUE) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"error training right node");
      return ISIP_FALSE;
    }

  }
  else if (l_or_t == TREE_TERM_INDICATOR) {
    // this line is a terminal
    //

    // setup the dataspace for the class information
    //
    if (class_d != (char_1**)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"class_d not NULL");
      return ISIP_FALSE;
    }
    if (class_count_d != (int_4*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"class_count_d not NULL");
      return ISIP_FALSE;
    }
    
    class_d = new char_1*[TREE_MAX_NUM_CLASSES];
    class_count_d = new int_4[TREE_MAX_NUM_CLASSES];

    if ((class_d == (char_1**)NULL) || (class_count_d == (int_4*)NULL)) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"out of memory");
      return ISIP_FALSE;
    }
    
    memset(class_d,0,TREE_MAX_NUM_CLASSES * sizeof(char_1*));
    memset(class_count_d,0,TREE_MAX_NUM_CLASSES * sizeof(int_4));
    
    
    // read in the class information
    //
    static char_1 class_str[ISIP_MAX_STRING_LENGTH];
    memset(class_str,0,ISIP_MAX_STRING_LENGTH);
    if(sscanf((char*)&line[level + 3], TREE_TERM_IMASK,
	      (char*)prefix, (char*)class_str) != 2) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"invalid terminal");
      return ISIP_FALSE;
    }
    if (debug_level_d > ISIP_DEBUG_DETAILED) {
      fprintf(stdout,"class = >%s<\n",class_str);
    }

    // tokenize the class information
    static char_1 temp_class_str[ISIP_MAX_STRING_LENGTH];
    char_1 *class_tok_str;
    int_4 i=1;

    class_tok_str = (char_1*)strtok((char*)class_str,TREE_CLASS_DELIM);

    if (sscanf((char*)class_tok_str,TREE_CLASS_FORMAT,&class_count_d[0],
	       (char*)temp_class_str) != 2) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"error reading class substring");
      return ISIP_FALSE;
    }
    if (class_d[0] != (char_1*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"class_d elem not NULL");
      return ISIP_FALSE;
    }     
    class_d[0] = new char_1[strlen((char*)temp_class_str) + 1];
    if (class_d[0] == (char_1*)NULL) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"out of memory");
      return ISIP_FALSE;
    }     

    strcpy((char*)class_d[0],(char*)temp_class_str);
    

    while ((class_tok_str = (char_1*)strtok((char*)NULL,TREE_CLASS_DELIM))
	   != (char_1*)NULL) {
      
      if (sscanf((char*)class_tok_str,TREE_CLASS_FORMAT,&class_count_d[i],
		 (char*)temp_class_str) != 2) {
	error_handler_cc((char_1*)"read_cc",
			 (char_1*)"error reading class substring");
	return ISIP_FALSE;
      }
      
      if (class_d[i] != (char_1*)NULL) {
	error_handler_cc((char_1*)"read_cc",
			 (char_1*)"class_d elem not NULL");
	return ISIP_FALSE;
      }     
      class_d[i] = new char_1[strlen((char*)temp_class_str) + 1];
      if (class_d[i] == (char_1*)NULL) {
	error_handler_cc((char_1*)"read_cc",
			 (char_1*)"out of memory");
	return ISIP_FALSE;
      }
	 
      strcpy((char*)class_d[i],(char*)temp_class_str);
      
      i++;
    }
    
  }
  else {
    error_handler_cc((char_1*)"read_cc",
		     (char_1*)"invalid level/terminal indicator");
    return ISIP_FALSE;
  }
  
  
  // now lets worry about the probability
  //
  if (strlen((char*)prefix_a) == 0) {
    // this must be the root node, make sure the level is zero and the
    // prefix read in is has nothing in it.
    //
    if (level_a != 0) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"no prob info for non-root node");
      return ISIP_FALSE;
    }
    
  }
  else {
    if (sscanf((char*)prefix, (char*)prefix_a, &prob_a) != 1) {
      error_handler_cc((char_1*)"read_cc",
		       (char_1*)"error reading probability");
      return ISIP_FALSE;
    }
  }
  
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
