// file: tree_prnt_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "tree.h"

#include <integral_constants.h>
#include "tree_constants.h"

// method: write_cc
//
// arguments:
//  FILE* fp_a: (input) file pointer
//  int_4 level_a: (input) current tree level
//  char_1* prefix_a: (input) prefix string, holding probability
//
// return:
//  a logical_1 indicating status
//
//
logical_1 Tree::write_cc(FILE* fp_a, int_4 level_a, char_1* prefix_a) {

  if (fp_a == (FILE*)NULL) {
    error_handler_cc((char_1*)"print_cc",
		     (char_1*)"no file opened");
    return ISIP_FALSE;
  }

  if (prefix_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"print_cc",
		     (char_1*)"no prefix");
    return ISIP_FALSE;
  }
  
  char_1 spacer[level_a*2 + 1];
  char_1 next_prefix[TREE_PREFIX_LEN];
  
  memset((char_1*)spacer,(int_2)' ',level_a * 2 * sizeof(char_1));
  spacer[level_a*2] = (char_1)NULL;



  if ((left_child_d == (Tree*)NULL) && (right_child_d == (Tree*)NULL)) {

    if (class_d == (char_1**)NULL) {
      error_handler_cc((char_1*)"write_cc",
		       (char_1*)"class_d not set for terminal node");
    }
    if (class_count_d == (int_4*)NULL) {
      error_handler_cc((char_1*)"write_cc",
		       (char_1*)"class_count_d not set for terminal node");
      return ISIP_FALSE;
    }
    if (class_count_d == (int_4*)NULL) {
      error_handler_cc((char_1*)"write_cc",
		       (char_1*)"class_count_d not set for terminal node");
      return ISIP_FALSE;
    }
		       
				   
    // set up a string to hold the class information
    //
    static char_1 class_str[ISIP_MAX_STRING_LENGTH];
    static char_1 temp_class_str[ISIP_MAX_STRING_LENGTH];
    memset(class_str,0,ISIP_MAX_STRING_LENGTH);
    logical_1 class_flag = ISIP_FALSE;
    for(int_4 i=0; i<TREE_MAX_NUM_CLASSES; i++) {
      if (class_count_d[i] > 0) {
	sprintf((char*)temp_class_str,TREE_CLASS_FORMAT,class_count_d[i],
		(char*)class_d[i]);
	strcat((char*)class_str,(char*)temp_class_str);
	strcat((char*)class_str,TREE_CLASS_DELIM);
	class_flag = ISIP_TRUE;
      }
    }
    if (class_flag == ISIP_TRUE) {
      class_str[strlen((char*)class_str) - 1] = (char_1)NULL;
    }

    fprintf(fp_a,TREE_TERM_OMASK, spacer, level_a,
	    (char*)prefix_a, (char*)class_str);
  }
  else if ((left_child_d != (Tree*)NULL) && (right_child_d != (Tree*)NULL)) {

    fprintf(fp_a, TREE_LEVEL_OMASK, (char*)spacer,
	    level_a, prefix_a, pos_d, char_d);

    sprintf((char*)next_prefix,TREE_L_PROB_OMASK,left_prob_d);
    left_child_d->write_cc(fp_a, level_a+1, next_prefix);

    sprintf((char*)next_prefix,TREE_R_PROB_OMASK,right_prob_d);
    right_child_d->write_cc(fp_a, level_a+1, next_prefix);
    
  }
  else {
    error_handler_cc((char_1*)"print_cc",
		     (char_1*)"one node was null, the other wasn't");
    return ISIP_FALSE;
  }
		   


  // exit gracefully
  //
  return ISIP_TRUE;
}
