// file: tree_quest_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "tree.h"

#include <integral_constants.h>
#include "tree_constants.h"

// method: formulate_question_cc
//
// arguments:
//  logical_1 do_split_a: (output) whether to do the split or not
//  char_1** data_a: (input) indicating data
//  int_4 count_a: (input) indicating number of data
//  int_4 context_a: (input) indicating the context of data
//
// return:
//  a logical_1 indicating status
//
//
logical_1 Tree::formulate_question_cc(logical_1& do_split_a,
				      char_1** data_a, int_4 count_a,
				      int_4 context_a) {


  // loop indices
  //
  int_4 i, j, k;
  

  // check parameters
  //
  if (count_a < 1) {
    error_handler_cc((char_1*)"formulate_question_cc",
		     (char_1*)"invalid parameter: count_a");
    return ISIP_FALSE;
  }

  if (context_a < 1) {
    error_handler_cc((char_1*)"formulate_question_cc",
		     (char_1*)"invalid parameter: context_a");
    return ISIP_FALSE;
  }
    
  if (data_a == (char_1**)NULL) {
        error_handler_cc((char_1*)"formulate_question_cc",
		     (char_1*)"invalid parameter: data_a");
    return ISIP_FALSE;
  }

  for (j=0;j<count_a;j++) {
    if (data_a[j] == (char_1*)NULL) {
      error_handler_cc((char_1*)"formulate_question_cc",
		       (char_1*)"invalid parameter: data_a");
      return ISIP_FALSE;
    }
  }
  
  // this static makes it so so much more efficient - RJD
  //
  static int_4 count_matrix[TREE_NUM_CHARS][ISIP_MAX_STRING_LENGTH];
  memset(count_matrix,0,
	 TREE_NUM_CHARS * ISIP_MAX_STRING_LENGTH * sizeof(int_4));
  
  // define and initialize matrix to keep max count
  //
  static int_4 max_counts[ISIP_MAX_STRING_LENGTH];
  static char_1 max_counted_char[ISIP_MAX_STRING_LENGTH];
  
  memset(max_counts, (int_2)0, ISIP_MAX_STRING_LENGTH * sizeof(int_4));
  memset(max_counted_char, (int_2)0,
	 ISIP_MAX_STRING_LENGTH * sizeof(char_1));
  
  // find the count of each symbol in all positions
  //
  for (i=0; i<context_a; i++) {
    for (j=0; j<count_a; j++) {
      for (k=0; k<TREE_NUM_CHARS; k++) {
	if (data_a[j][i] == chars[k]) {
	  count_matrix[k][i]++;
	}      
      }
    }
  }
  
  // do a sanity check...make sure that the sum of count_matrix
  // accross each row is equal to the count_a
  //
  int_4 temp_count;
  for (i=0; i<context_a; i++) {
    temp_count = count_a;
    for(k=0;k<TREE_NUM_CHARS;k++) {
      temp_count -= count_matrix[k][i];
    }
    if (temp_count > 0) {
      error_handler_cc((char_1*)"formulate_question_cc",
		       (char_1*)"sanity check for sum failed");
    }		       
  }

  // find the maximum occurence of a given symbol with respect to the rest
  //
  for (i=0; i<context_a; i++) {

    // find the maximum for this position in the context
    //
    max_counts[i] = TREE_MIN_COUNT_SIMILAR;
    for (k=0; k<TREE_NUM_CHARS; k++) {
      if (count_matrix[k][i] > max_counts[i]) {
	max_counted_char[i] = chars[k];
	max_counts[i] = count_matrix[k][i];
      }
    }
  }

  // now find the maximum difference; this is the maximum split
  //
  int_4 max_diff = TREE_MIN_COUNT_SIMILAR;
  
  for (i=0; i<context_a; i++) {
    if ((count_a - max_counts[i]) > max_diff) {
      max_diff = count_a - max_counts[i];
      char_d = max_counted_char[i];
      pos_d = i;
    }
  }

  if (max_diff == TREE_MIN_COUNT_SIMILAR) {
    // the data is identical, we cannot split it again
    //
    do_split_a = ISIP_FALSE;
  }
  else {
    // split the data
    //
    do_split_a = ISIP_TRUE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}







