// file: tree_test_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "tree.h"

#include <integral_constants.h>
#include "tree_constants.h"

// method: test_cc
//
// arguments:
//  char_1** classes_a: (output) classes of contexts
//  int_4 nbest_a: (input)
//  float_4* probs_a: (output) indicating the probability of a given context
//  char_1** attrs_a: (input) contexts
//  int_4 context_a: (input) indicating the context of data
//
// return:
//  a logical_1 indicating status
//
//
logical_1 Tree::test_cc(FILE* fp_a, int_4 nbest_a, char_1* attr_a,
			char_1* class_a, float_4 prob_a) {

  // check parameters
  //
  if (fp_a == (FILE*)NULL) {
    error_handler_cc((char_1*)"test_cc",
		     (char_1*)"null file pointer: fp_a");
    return ISIP_FALSE;
  }

  Tree* tree_ptr = this;
  tree_ptr->prob_d = prob_a;
  
  while ((tree_ptr->left_child_d != (Tree*)NULL) &&
	 (tree_ptr->right_child_d != (Tree*)NULL)) {
    
    // depending on how the question matches, go either to the right
    // or to the left
    //
    if (tree_ptr->match_question_cc(attr_a) == ISIP_TRUE) {
      
      // multiplicatively combine the probability
      //
      prob_a *= tree_ptr->left_prob_d;
        
      // increment the counter
      //
      tree_ptr = tree_ptr->left_child_d;
      tree_ptr->prob_d = prob_a;
  }

    else {
      
      // multiplicatively combine the probability
      //
      prob_a *= tree_ptr->right_prob_d;
      
      // increment the counter
      //
      tree_ptr = tree_ptr->right_child_d;
      tree_ptr->prob_d = prob_a;
    }
  }

  // output the classes at this terminal node
  //
  int_4 max_a = TREE_MAX_PHONEMES;
  int_4 count_a = 0;
  int_4 total = 0;
  for (int_4 i=0; i<TREE_MAX_NUM_CLASSES; i++) {
    total += tree_ptr->class_count_d[i];
  }
  for (int_4 i=0; i<TREE_MAX_NUM_CLASSES; i++) {
    if ((count_a < max_a) && (tree_ptr->class_d[i] != (char_1*)NULL)) {
      strcpy((char*)class_a, (char*)tree_ptr->class_d[i]);
      fprintf(fp_a, "%s\t\t%s\t\t%f\n", attr_a, class_a,
	      prob_a*(float_4)(tree_ptr->class_count_d[i])/total);
      count_a++;
      
      if (debug_level_d > ISIP_DEBUG_BRIEF) {
	fprintf(stdout, "%s\t\t%s\t\t%f\n", attr_a, class_a,
		prob_a*(float_4)(tree_ptr->class_count_d[i])/total);
      }
    }
    else {
      break;
    }
  }
    
  // trace up the tree to the sibling node if required
  //
  if (count_a < max_a) {
    Tree* sibling_ptr = tree_ptr;
    if (sibling_ptr == (sibling_ptr->parent_d)->left_child_d) {
      sibling_ptr = (sibling_ptr->parent_d)->right_child_d;
      sibling_ptr->prob_d = (sibling_ptr->parent_d)->prob_d *
	(sibling_ptr->parent_d)->right_prob_d;
    }
    else {
      sibling_ptr = (sibling_ptr->parent_d)->left_child_d;
      sibling_ptr->prob_d = (sibling_ptr->parent_d)->prob_d *
	(sibling_ptr->parent_d)->left_prob_d;
    }

    // generate more outputs
    //
    traverse_cc(sibling_ptr, count_a, max_a, fp_a, attr_a, class_a, sibling_ptr->prob_d);
  }
  
  // go up the tree and then look at alternative paths if more phones are
  // wanted
  //
  if (count_a < max_a) {
    int_4 upstep = (int_4)(log(nbest_a)/log(2.0));
    Tree* grandpa_ptr = tree_ptr;
    while ((upstep > 0) && (grandpa_ptr->parent_d != (Tree*)NULL)) {
      if (grandpa_ptr == (grandpa_ptr->parent_d)->left_child_d) {
	grandpa_ptr = grandpa_ptr->parent_d;
	grandpa_ptr->prob_d *= grandpa_ptr->left_prob_d;
      }
      else {
	grandpa_ptr = grandpa_ptr->parent_d;
	grandpa_ptr->prob_d *= grandpa_ptr->right_prob_d;
      }
      upstep --;
    }
    
    // generate requisite outputs
    //
    traverse_cc(grandpa_ptr, count_a, max_a, fp_a, attr_a, class_a, grandpa_ptr->prob_d);
    for (int_4 l = count_a; l < max_a; l++) {
      
      // output the class at terminal node if all phones are not yet found
      //
      strcpy((char*)class_a, (char*)tree_ptr->class_d[0]);
      fprintf(fp_a, "%s\t\t%s\t\t%f\n", attr_a, class_a, prob_a);
      
      if (debug_level_d > ISIP_DEBUG_BRIEF) {
	fprintf(stdout, "%s\t\t%s\t\t%f\n", attr_a, class_a, prob_a);
      }
    }    
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}

