// file: tree_test_1.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include <integral_constants.h>
#include "tree.h"
#include "tree_constants.h"

// method: traverse_cc
//
// arguments:
//  char_1** classes_a: (output) classes of contexts
//  int_4 nbest_a: (input)
//  float_4* probs_a: (output) indicating the probability of a given context
//  char_1** attrs_a: (input) contexts
//  int_4 context_a: (input) indicating the context of data
//
// return:
//  a logical_1 indicating status
//
//
logical_1 Tree::traverse_cc(Tree* node_a, int_4& count_a, int_4 nbest_a,
			    FILE* fp_a, char_1* attr_a, char_1* class_a,
			    float_4 prob_a) {
  
  // return if n-best is already achieved
  //
  if (count_a == nbest_a) {
    return ISIP_TRUE;
  }
  
  // start at the top node
  //
  if (node_a != (Tree*)NULL) {
    traverse_cc(node_a->left_child_d, count_a, nbest_a, fp_a, attr_a,
		class_a, prob_a*node_a->left_prob_d);
    if ((node_a->left_child_d == (Tree*)NULL) &&
	(node_a->right_child_d == (Tree*)NULL)) {
      
      int_4 total = 0;
      for (int_4 i=0; i<TREE_MAX_NUM_CLASSES; i++) {
	total += node_a->class_count_d[i];
      }
      
      // output the classes at this terminal node
      //
      for (int_4 i=0; i<TREE_MAX_NUM_CLASSES; i++) {
	if ((count_a < nbest_a) && (node_a->class_d[i] != (char_1*)NULL)) {
	  strcpy((char*)class_a, (char*)node_a->class_d[i]);
	  fprintf(fp_a, "%s\t\t%s\t\t%f\n", attr_a, class_a,
		  prob_a*(float_4)(node_a->class_count_d[i])/total);
	  count_a++;
	  
	  if (debug_level_d > ISIP_DEBUG_BRIEF) {
	    fprintf(stdout, "%s\t\t%s\t\t%f\n", attr_a, class_a,
		    prob_a*(float_4)(node_a->class_count_d[i])/total);
	  }
	}
	else {
	  break;
	}
      }
    }
    traverse_cc(node_a->right_child_d, count_a, nbest_a, fp_a, attr_a,
		class_a, prob_a*node_a->right_prob_d);
  }
  
  // return
  //
  return ISIP_TRUE;
}



