// file: tree_train_1.cc
//

// isip include files
//
#include "tree.h"

#include <integral_constants.h>
#include "tree_constants.h"

// method: train_right_cc
//
// arguments:
//  char_1** attrs_a: (input) indicating context
//  char_1** classes_a: (input)indicating the class of each context
//  int_4 count_a: (input) indicating number of data
//  int_4 context_a: (input) indicating the context of data
//
// return:
//  a logical_1 indicating status
//
//

int_4 Tree::train_right_cc(char_1** attrs_a, char_1** classes_a,
			  int_4 count_a, int_4 context_a) {

  // check parameters
  //
  if (count_a < 1) {
    error_handler_cc((char_1*)"train_right_cc",
		     (char_1*)"invalid parameter: count_a");
    return ISIP_FALSE;
  }

  if (context_a < 1) {
    error_handler_cc((char_1*)"train_right_cc",
		     (char_1*)"invalid parameter: context_a");
    return ISIP_FALSE;
  }
    
  if (attrs_a == (char_1**)NULL) {
        error_handler_cc((char_1*)"train_right_cc",
			 (char_1*)"invalid parameter: attrs_a");
    return ISIP_FALSE;
  }

  if (classes_a == (char_1**)NULL) {
    error_handler_cc((char_1*)"train_cc",
		     (char_1*)"invalid parameter: classes_a");
    return ISIP_FALSE;
  }

  for (int_4 j=0;j<count_a;j++) {
    if (attrs_a[j] == (char_1*)NULL) {
      error_handler_cc((char_1*)"train_right_cc",
		       (char_1*)"invalid parameter: attrs_a");
      return ISIP_FALSE;
    }
    if (classes_a[j] == (char_1*)NULL) {
      error_handler_cc((char_1*)"train_cc",
		       (char_1*)"invalid parameter: classes_a");
      return ISIP_FALSE;
    }

  }
    
  // temp variables to process data
  //
  int_4 temp_count=0;
  
  // count the number of yes's to the question
  //
  for(int_4 j=0; j<count_a; j++) {
    if (match_question_cc(attrs_a[j]) != ISIP_TRUE) {
      temp_count++;
    }
  }

  // see if we need to further divide this tree
  //
  if (temp_count > 0) {
    
    // allocate the data to be passed to the right node
    //
    int_4 temp_right_count=0;
    char_1* right_attrs[temp_count];
    char_1* right_classes[temp_count];
    
    for(int_4 j=0; j<count_a; j++) {
      if (match_question_cc(attrs_a[j]) != ISIP_TRUE) {
	right_attrs[temp_right_count] = attrs_a[j];
	right_classes[temp_right_count++] = classes_a[j];
      }
    }

    // try to fight off memory leaks
    //
    if (right_child_d != (Tree*) NULL) {
      error_handler_cc((char_1*)"train_right_cc",
		       (char_1*)"right_child_d not NULL");
      return ISIP_FALSE;
    }

    right_child_d = new Tree(fatal_mode_d);
    if (right_child_d == (Tree*)NULL) {
      error_handler_cc((char_1*)"train_right_cc",
		       (char_1*)"out of memory");
      return ISIP_FALSE;
    }
    right_child_d->train_cc(this, right_attrs, right_classes, temp_count,
			   context_a);

  }
  else {
    error_handler_cc((char_1*)"train_right_cc",
		     (char_1*)"all data matched the question");
    return ISIP_FALSE;
  }

  // exit gracefully
  //
  return temp_count;
  
}


