// file: boltz_constants.h
//
// declaration of the global constants used in the boltzmann machine
//

// make sure definitions are only made once
//
#ifndef __ISIP_BOLTZ_CONSTANTS
#define __ISIP_BOLTZ_CONSTANTS

// system include files
//
#include <assert.h>
#include <string.h>
#include <ctype.h>

// isip include files
//
#include "integral_constants.h"

//-----------------------------------------------------------------------------
//
// general constants
//
//-----------------------------------------------------------------------------

// define some maximum string sizes
//
#define MAX_STRING_SIZE              1024
#define MAX_WORD_SIZE                32
#define MAX_PHONE_SIZE               3

// define some parameters used in testing the machine
//
#define DEFAULT_NBEST                5
#define DEFAULT_NUM_ATTEMPTS         5

// flag to indicate functioning mode of the boltzmann machine
//
#define BOLTZ_TRAIN                  0
#define BOLTZ_TEST                   1
#define MLP_TEST                   2


// flag to indicate training status of the boltzmann machine
//
#define BOLTZ_UNTRAINED              ISIP_FALSE
#define BOLTZ_TRAINED                ISIP_TRUE

// define some training parameters
//
#define DEFAULT_RAND_LIMIT           0.1
#define DEFAULT_MAX_EPOCHS           10000
#define DEFAULT_MAX_ERROR            1.0E-5
#define DEFAULT_INIT_TEMPERATURE     1.0
#define DEFAULT_MIN_TEMPERATURE      1.0E-5
#define DEFAULT_FINAL_MOMENTUM       0.95
#define DEFAULT_LEARNING_RATE        0.35
#define DEFAULT_EXPONENT             0.05

// flag to indicate contexts
//
#define BOLTZ_NONE                   0
#define BOLTZ_SHORT                  1
#define BOLTZ_LONG                   2
#define BOLTZ_BOTH                   3

// define sizes of contexts used in training the machine
//
#define SHORT_CONTEXT_SIZE           3
#define LONG_CONTEXT_SIZE            7

// define terms to do weighted updating
//
#define MAX_BIT_ERROR                0.3
#define UPDATE_SCALE_FACTOR          1.25

// end of file
//
#endif
