// file: boltz_neuron.h
//
 
// make sure definitions are only made once
//
#ifndef __ISIP_BOLTZ_NEURON
#define __ISIP_BOLTZ_NEURON

// isip include files
//
#ifndef __ISIP_BOLTZ_CONSTANTS
#include <boltz_constants.h>
#endif

#ifndef __ISIP_BOLTZ_RANDOM
#include <boltz_random.h>
#endif

// Boltz_neuron: the neuron class that constitutes the hidden and 
// output neuron layers in the Boltzmann machine
//
class Boltz_neuron {

  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:
  
  // neuron parameter data
  //
  float_4 output_d;		// output of fired neuron
  float_4 energy_gap_d;         // energy gap -ve Delta E
  float_4 probability_d;        // probability of firing
  static float_4 inv_temp_d;    // the inverse of the system temperature
  
  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // constructors and destructor
  //
  ~Boltz_neuron ();                 // destructor
  Boltz_neuron ();                  // default constructor
  
  // output methods
  //
  logical_1 output_cc (float_4 &out_a);       // get the neuron output
  logical_1 likelihood_cc (float_4 &lhood_a); // get the firing probability
  
  // internal calculation methods
  //
  logical_1 set_energy_cc (float_4** weights_a, float_4* inputs_a, 
			   int_4 length_a, int_4 col_a);
  logical_1 set_energy_both_cc (float_4** weights_short_a, 
				float_4* inputs_short_a, int_4 length_short_a,
				float_4** weights_long_a, 
				float_4* inputs_long_a, int_4 length_long_a, 
				int_4 col_a);
  logical_1 set_output_cc (logical_1 function_a);
  static logical_1 set_inv_temp_cc (float_4 itmp_a);
};

// end of file
//
#endif
