// file:: Input_buffer.h
//

// make sure definitions are only made once
//
#ifndef __ISIP_INPUT_BUFFER
#define __ISIP_INPUT_BUFFER

// isip include files
//
#ifndef __ISIP_BOLTZ_CONSTANTS
#include <boltz_constants.h>
#endif

// Input_buffer: an input register class that makes up the 
// input layer of the Boltzmann machine
//
class Input_buffer {

  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:
  
  // shift register parameter data
  //
  int_4 in_size_d;               // number of input characters
  int_4 out_size_d;              // number of output bits
  int_4 num_input_bits_d;        // number of bits per input character
  char_1* input_d;               // array of input characters
  float_4* output_d;             // array of output bits
  char_1* name_d;                // the name stored in the register
  int_4 name_length_d;           // the length of the name string
  int_4 index_d;                 // the index of letters in the register

  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // constructors and destructor
  //
  ~Input_buffer ();                  // destructor
  Input_buffer (int_4 size_a);       // default constructor

  // input / output methods
  //
  logical_1 load_cc (char_1* name_a);      // load new word
  logical_1 output_cc (float_4* out_a);    // get the output bits
  logical_1 shift_cc ();                   // shift contents 
  logical_1 empty_cc ();                   // check if register is empty

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  
  // internal calculation methods
  //
  logical_1 clear_cc ();
  logical_1 set_outs_cc (int_4 pos_a);
  logical_1 set_bits_cc (float_4* bits_a, char_1 symbol_a);
};

// end of file
//
#endif
