// file: output_pron.h
//

// make sure definitions are only made once
//
#ifndef __ISIP_OUTPUT_PRON
#define __ISIP_OUTPUT_PRON
 
// isip include files
//
#ifndef __ISIP_BOLTZ_CONSTANTS
#include <boltz_constants.h>
#endif

// Output_pron: the string class that stores and sorts the output
// pronunciations of the Boltzmann machine according to the likelihood 
// scores
//
class Output_pron {
  
  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:

  // pronunciations parameter data
  //
  char_1* name_spell_d;           // spelling of the name
  char_1** name_prons_d;          // array of phoetic pronunciation strings
  float_4* pron_score_d;          // array of scores related to pronunciations
  int_4 num_prons_d;              // number of pronunciation strings
  int_4 num_uniq_prons_d;         // number of unique pronunciation strings
  int_4 name_length_d;            // length of the name string
  float_4 scale_log_d;            // standard averaging factor for scores

  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // constructor and destructor
  //
  ~Output_pron ();
  Output_pron (char_1* name_a, int_4 num_prons_a);

  // add a pronunciation to array
  //
  logical_1 add_pron_cc (char_1* pron_a, float_4 score_a); //

  // print list of pronunciations to file
  //
  logical_1 print_prons_cc (FILE* outfile_a);

  // find number of unique pronunciations
  //
  logical_1 uniq_prons_cc (int_4 &num_uniq_a);

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

  // sort the pronunciations according to scores
  //
  logical_1 sort_prons_cc ();
};

// end of file
//
#endif
