// file: tree.h
//

// make sure definitions are made only once
//
#ifndef __ISIP_TREE
#define __ISIP_TREE

// isip include files
//
#ifndef __ISIP_INTEGRAL
#include <integral.h>
#endif

#ifndef __ISIP_INTEGRAL_CONSTANTS
#include <integral_constants.h>
#endif

// Tree: a class to handle a decision tree
//
class Tree {
  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:

  // declare a variable for debugging
  //
  int_4 debug_level_d;
  logical_1 fatal_mode_d;

  // internal data
  //
  char_1 char_d;        // part of question of what character to split and at
  int_4 pos_d;          // what position in the context
  
  float_8 left_prob_d;  // probability if answer to question is yes
  float_8 right_prob_d; // probability if answer to question is no
  float_8 prob_d;       // probability of being in this node
  
  char_1** class_d;
  int_4* class_count_d;
  
  Tree* parent_d;  
  Tree* left_child_d;
  Tree* right_child_d;
  
  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // for debug
  //
  static int_4 counter;
  
  // required methods
  //
  char_1* name_cc();
  logical_1 error_handler_cc(char_1* method_name, char_1* message);
  logical_1 debug_cc(FILE* fp, char_1* message);
  logical_1 set_debug_cc(int_4 debug_level);
  
  // destructor/constructor 
  //
  ~Tree();
  Tree();
  Tree(logical_1 fatal_mode);

  // train methods
  //
  logical_1 train_cc(Tree* parent, char_1** attrs, char_1** classes,
		     int_4 count, int_4 context);

  logical_1 formulate_question_cc(logical_1 &split, char_1** attrs,
				  int_4 count, int_4 context);

  int_4 train_left_cc(char_1** attrs, char_1** classes,
		      int_4 count, int_4 context);
  int_4 train_right_cc(char_1** attrs, char_1** classes,
		       int_4 count, int_4 context);

  logical_1 match_question_cc(char_1* string);
  
  // test methods
  //
  logical_1 test_cc(FILE* fname, int_4 nbest, char_1* attr, char_1* cl,
		    float_4 prob);

  logical_1 traverse_cc(Tree* node, int_4& count, int_4 nbest, FILE* fp,
			char_1* attr, char_1* cl, float_4 prob);
  
  // io methods -- high level interface (these should be called directly)
  //
  logical_1 write_cc(FILE* fp);
  logical_1 read_cc(FILE* fp);

  // low level recursive io methods...don't call these directly
  //
  logical_1 write_cc(FILE* fp, int_4 level, char_1* prefix);
  logical_1 read_cc(float_8& prob, FILE* fp, int_4 level, char_1* prefix);
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

  // exit handler methods
  //
  volatile void exit_handler_cc();

};

// end of file
//
#endif






