// file: tree_constants.h
//
// this file defines global paths, filenames, and constants
// of the Tree class.
//
 
// make sure definitions are made only once
//
#ifndef __ISIP_TREE_CONSTANTS
#define __ISIP_TREE_CONSTANTS
 
// isip include files
//
#ifndef __ISIP_INTEGRAL
#include <integral.h>
#endif

// define the class name
//
#define TREE_CLASS_NAME (char_1*)"Tree"

// define number of phonetic symbols
//
#define TREE_NUM_CHARS (int_4)31

// define a maximum number of levels, useful for printing
//
#define TREE_MAX_LEVELS 8192

// define the length of the string that holds the prefix
//
#define TREE_PREFIX_LEN 64

// define the minimum number of splits
//
#define TREE_MIN_COUNT_SIMILAR 0

// define the maximum number of phones to be output per letter context
//
#define TREE_MAX_PHONEMES 2

// define the set of input characters
//
static const char_1 chars[TREE_NUM_CHARS] = {
  '_', ' ', '-', '\'', '.', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z'
};

#define TREE_PHONE_SEPARATOR ","

// define maximum number of classes (phones) per leaf node
//
#define TREE_MAX_NUM_CLASSES TREE_NUM_CHARS

// define constants used for reading and writing the class info
//
#define TREE_CLASS_DELIM ","
#define TREE_CLASS_FORMAT "%ld %s"

// define some codes for the tree file
//
#define TREE_LEVEL_INDICATOR 'L'
#define TREE_TERM_INDICATOR 'T'

#define TREE_L_PROB_OMASK "yes = %5.5f "
#define TREE_R_PROB_OMASK "no  = %5.5f "

#define TREE_L_PROB_IMASK "%*[^y]yes = %lf"
#define TREE_R_PROB_IMASK "%*[^n]no  = %lf"

#define TREE_LNUM_MASK "%c%2ld"

#define TREE_LEVEL_IMASK "%[^I]Is (str[%ld] == %c) ?"
#define TREE_TERM_IMASK "%[^C]Class: %[^\n]"

#define TREE_LEVEL_IMASK_SPACE_CHECK ')'
#define TREE_LEVEL_OMASK "%sL%2.2ld %sIs (str[%ld] == %c) ?\n"
#define TREE_TERM_OMASK "%sT%2.2ld %sClass: %s\n"

// end of file
//
#endif
