// file: build_net.cc
//
// main function to generate a network of phonemes from the
// n-best pronunciations
//

// isip include files
//
#include "array.h"

//-----------------------------------------------------------------------------
// method: main
//
// main function for building pronunciation network file
//
//-----------------------------------------------------------------------------
main (int_4 argc, char_1** argv) {
  
  // verify arguments
  //
  if (argc != 4) {
    fprintf (stderr, "Usage: build_net <infile> <outfile> <n_best>\n");
    exit (1);
  }
  
  // open the input file and create a network
  //
  char_1* infile_d = argv[1];
  char_1* outfile_d = argv[2];
  FILE* inFile_d = fopen ((char*)infile_d, "r");
  if (inFile_d == NULL) {
    fprintf (stderr, "Error in opening file %s.\n", infile_d);
    exit (ISIP_ERROR);
  }
  
  // read the data from the file
  //
  char_1* data_str_d  = new char_1[MAX_WORD_SIZE];
  char_1* phone_str_d = new char_1[MAX_PHONE_SIZE];
  
  // read the header
  //
  fscanf (inFile_d, "%[* ]", data_str_d);
  fscanf (inFile_d, "%[^\n]", data_str_d);
  int_4 word_length_d = strlen ((char*)data_str_d);

  // create the arrays
  //
  int_4 n_best_d = (int_4) atoi ((char*)argv[3]);
  Array **phone_matrix_d = new Array*[n_best_d];
  for (int_4 j = 0; j < n_best_d; j++) {
    phone_matrix_d[j] = new Array (j, word_length_d);
  }
  
  // read the phonemes one by one and store them in arrays
  //
  int_4 i;
  int_4 num_rows_d = 0;
  int_4 blank_rows_d = 0;
  int_4 count_rows_d = 0;
  char_1* temp_str_d = new char_1[MAX_PHONE_SIZE];
  
  while (fscanf (inFile_d, "%[\n ]", data_str_d) != EOF) {
    
    // read the phonemes
    //
    for (i = 0; i < word_length_d; i++) {
      
      if (fscanf (inFile_d, "%s", phone_str_d) == EOF) {
	break;
      }
      
      // check and insert phoneme in the appropriate array
      //
      count_rows_d = 0;
      blank_rows_d = 0;
      for (int_4 j = 0; j < num_rows_d + 1; j++) {
	if (phone_matrix_d[j]->check_phone_cc (i, phone_str_d) == ISIP_FALSE) {
	  if (phone_matrix_d[j]->check_phone_cc (i, (char_1*)"#") == ISIP_FALSE) {
	    count_rows_d ++;
	  }
	  else {
	    blank_rows_d ++;
	  }
	}
      }
      
      // now find a suitable place to insert this
      //
      if (count_rows_d + blank_rows_d == num_rows_d + 1) {
	int_4 j = 0;
	for (j = 0; j < num_rows_d + 1; j++) {
	  if (phone_matrix_d[j]->check_phone_cc (i, (char_1*)"#") == ISIP_TRUE) {
	    phone_matrix_d[j]->set_phone_cc (i, phone_str_d);
	    break;
	  }

	  else if (j == num_rows_d) {
	    phone_matrix_d[++ num_rows_d]->set_phone_cc (i, phone_str_d);
	    break;
	  }
	}
      }
    }

    // read junk after the phones
    //
    if (fscanf (inFile_d, "%[^\n]", data_str_d) == EOF)
      break;
  }
  
  // reset the blanks
  //
  for (int_4 nn = 0; nn < word_length_d; nn++)  {
    for (int_4 ii = 1; ii < num_rows_d + 1; ii++) {
      if (phone_matrix_d[ii]->check_phone_cc (nn, (char_1*)"_") == ISIP_TRUE) {
	phone_matrix_d[0]->get_phone_cc (nn, temp_str_d);
	phone_matrix_d[ii]->set_phone_cc (nn, temp_str_d);
	phone_matrix_d[0]->set_phone_cc (nn, (char_1*)"_");
      }
    }
  }
  
  // re-align the blanks
  //
  for (int_4 nn = 0; nn < word_length_d; nn++) {
    for (int_4 ii = 1; ii < num_rows_d + 1; ii++) {
      if (phone_matrix_d[0]->check_phone_cc (nn, (char_1*)"_") == ISIP_TRUE) {
	if (phone_matrix_d[ii]->check_phone_cc (nn, (char_1*)"#") == ISIP_TRUE) {
	  phone_matrix_d[ii]->set_phone_cc (nn, (char_1*)"_");
	}
      }
    }
  }

  // open file for output
  //
  FILE *outFile_d = fopen ((char*)outfile_d, "w");
  if (outFile_d == NULL) {
    fprintf (stderr, "Error in opening file %s.\n", outfile_d);
    exit (ISIP_ERROR);
  }
  
  // write the arrays to output
  //
  for (int_4 ii = 0; ii < num_rows_d + 1; ii++) {
    phone_matrix_d[ii]->print_array_cc (outFile_d);
  }
  
  // free memory
  //
  fclose (inFile_d);
  fclose (outFile_d);
  delete [] phone_str_d;
  delete [] data_str_d;
  delete [] temp_str_d;
  delete [] phone_matrix_d;
}
