// file: generate_pron.cc
// 
// main function for training and executing a boltzmann machine neural network
//

// isip include files
//
#include "generate_pron.h"

//-----------------------------------------------------------------------------
//method: main
//
// main function for boltzmann machine
//
//-----------------------------------------------------------------------------
main (int_4 argc, char_1** argv) {

  // various string variables for options
  //
  char_1* operation_l = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* prmsfile_l = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* datafile_l = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* outfile_l = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* wtfile_l = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // dummy variables
  //
  int_4 nbest_l = 0;
  
  // check for commandline inputs being correct
  //
  for (int_4 ind = 0; ind < argc; ind ++) {
    
    // check for the help message
    //
    if ((argc == 1) || (strcmp((char*)ISIP_HELP_OPTION,
			       (char*)argv[ind]) == 0)) {
      
      // print the help message
      //
      char_1 cmd_str[ISIP_MAX_STRING_LENGTH];
      sprintf((char*)cmd_str, (char*)ISIP_HELP_FORMAT, (char*)GP_HELP_FILE);
      system((char*)cmd_str);
      
      // exit gracefully
      //
      exit(ISIP_ERROR);
    }
    
    // determine function --- training or testing
    //
    else if (strcmp(GP_FUNCTION_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)operation_l, (char*)argv[ind+1]);
    }
    
    // read the name of the parameter file
    //
    else if (strcmp(GP_PARAMS_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)prmsfile_l, (char*)argv[ind+1]);
    }
    
    // read the number of options (n-best) wanted
    //
    if (strcmp(GP_NBEST_OPT, (char*)argv[ind]) == 0) {
      nbest_l = (int_4)atoi((char*)argv[ind+1]);
    }
    
    // read the name of the training / testing datafile
    //
    if (strcmp(GP_INFILE_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)datafile_l, (char*)argv[ind+1]);
    }
    
    // read the name of the output file
    // this will be the log file for training
    // or the pronunciations output file for testing
    //
    if (strcmp(GP_OUTFILE_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)outfile_l, (char*)argv[ind+1]);
    }
  }
  
  // create a Boltzmann machine network and train or run it as specified
  //
  Boltz_machine boltz_network_d;
  
  // load appropriate parameters
  //
  boltz_network_d.set_params_cc(wtfile_l, operation_l, prmsfile_l, nbest_l);
  
  // train the machine
  //
  if (strcmp((char*)operation_l, "train") == 0) {
    boltz_network_d.train_cc (outfile_l, wtfile_l, datafile_l);
  }
  
  // run the machine
  //
  else if ((strcmp((char*)operation_l, "test") == 0) ||
	   (strcmp((char*)operation_l, "mlp") == 0)) {
    boltz_network_d.test_cc (outfile_l, wtfile_l, datafile_l);
  }

  // free memory
  //
  delete [] operation_l;
  delete [] prmsfile_l;
  delete [] datafile_l;
  delete [] outfile_l;
  delete [] wtfile_l;
}

//
// end of file
