// file: $ISIP_PRON/src/decision_tree/v1.0/util/generate_tree/gtree_gp_0.cc
//
// a simple command line parser
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include "generate_tree.h"

#include <integral_constants.h>
#include "generate_tree_constants.h"

// function: get_parameters_cc
//
// arguments:
//  int_4& mode_a: (output) mode of action
//  int_4& context_a: (output) length of attributes
//  char_1* in_train_fname: (output) input filename
//  char_1* in_test_fname: (output) input filename
//  char_1* out_train_fname: (output) output filename
//  char_1* out_test_fname: (output) output filename
//  int_4& debug_level_a: (output) debug level
//  int_4 argc: (input) argc from the main program
//  char_1** argv: (input) argv from the main program
//
// return value: a logical_1 indicating status
//
// This function parses command line arguments.
//
//
logical_1 get_parameters_cc(int_4& mode_a, int_4& context_a, int_4& nbest_a,
			    char_1* in_fname_a, char_1* out_fname_a,
			    char_1* tree_fname_a, int_4& debug_level_a,
			    int_4 argc, char_1** argv) {
    
  // main decoding loop
  //
  int_4 last_index = argc;
  
  // if no arguments are given then print out a help message
  //
  if (last_index == (int)1) {
    char_1 tmp_string[ISIP_MAX_STRING_LENGTH];
    sprintf((char*)tmp_string,
	    (char*)ISIP_HELP_FORMAT, (char*)GTREE_HELP_FILE);
    system((char*)tmp_string);

    // exit gracefully
    //
    exit (ISIP_NO_ERROR);
  }
  
  for (int_4 index=1; index<last_index; index++) {

    // check for help
    //
    if (strcmp((char*)ISIP_HELP_OPTION, (char*)argv[index]) == (int)0) {
      char_1 tmp_string[ISIP_MAX_STRING_LENGTH];
      sprintf((char*)tmp_string,
	      (char*)ISIP_HELP_FORMAT, (char*)GTREE_HELP_FILE);
      system((char*)tmp_string);

      // exit gracefully
      //
      exit (ISIP_NO_ERROR);
    }

    // pick off mode of action
    //
    else if (strcmp((char*)GTREE_MODE_OPT, (char*)argv[index]) == (int)0) {
      mode_a = (int_4)atoi((char*)argv[++index]);
    }

    // pick off context (length of attributes assume all attributes are of
    // equal length)
    //
    else if (strcmp((char*)GTREE_CONTEXT_OPT, (char*)argv[index]) == (int)0) {
      context_a = (int_4)atoi((char*)argv[++index]);
    }

    // pick off nbest
    //
    else if (strcmp((char*)GTREE_NBEST_OPT, (char*)argv[index]) == (int)0) {
      nbest_a = (int_4)atoi((char*)argv[++index]);
    }

    // pick off debug option
    //
    else if (strcmp((char*)GTREE_DEBUG_OPT, (char*)argv[index]) == (int)0) {

      // save the debug level
      //
      debug_level_a = (int_4)atoi((char*)argv[++index]);
    }
    
    // pick off the filenames
    //
    else if (strcmp((char*)GTREE_INPUT_OPT, (char*)argv[index]) == (int)0) {
      strcpy((char*)in_fname_a, (char*)argv[++index]);
    }

    else if (strcmp((char*)GTREE_OUTPUT_OPT, (char*)argv[index]) == (int)0) {
      strcpy((char*)out_fname_a, (char*)argv[++index]);
    }

    // pick off the filename that contains the tree (for test mode only)
    //
    else if (strcmp((char*)GTREE_TREE_OPT, (char*)argv[index]) == (int)0) {
      strcpy((char*)tree_fname_a, (char*)argv[++index]);
    }
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}






