// file: $ISIP_PRON/src/decision_tree/v1.0/util/generate_tree/gtree_load_0.cc
//

// system include files
//
#include <stdio.h>
#include <string.h>

// isip include files
//
#include "generate_tree.h"

#include <integral_constants.h>
#include "generate_tree_constants.h"

// function: load_train_cc
//
// arguments:
//  char_1**& attributes_a: (output) buffer containing context
//  char_1**& classes_a:: (output) buffer containing phone
//  int_4& num_item_a: (output) number of items in the buffer
//  int_4 array_size_a: (input) size of buffer
//  char_1* in_fname_a: (input) name of file from which to read data
//  int_4 debug_level_a: (input) debug level
//
// return:
//  logical_1 to indicate status
//
// This function loads in the attributes and the classes from a given
// text file into buffers.  If the file doesn't exist, function outputs
// error message and exits.
//
//
logical_1 load_train_cc(char_1**& attributes_a, char_1**& classes_a,
			int_4& num_items_a, int_4& array_size_a,
			char_1* in_fname_a, int_4 debug_level_a) {

  // open file for reading
  //
  FILE* fp = fopen((char*)in_fname_a, "r");
    
  // check if there is an error in opening the file
  //
  if (fp == (FILE*)NULL) {
    fprintf(stdout, "cannot open file %s\n", in_fname_a);
    return ISIP_FALSE;
  }

  // allocate initial space
  //
  num_items_a = (int_4)0;
  array_size_a = (int_4)GTREE_INITIAL_ARRAY_SIZE;
  attributes_a = new (char_1*)[array_size_a];
  classes_a = new (char_1*)[array_size_a];

  // read the data into the buffer
  //
  char_1 tmp_buff[ISIP_MAX_STRING_LENGTH];
  while (fgets((char*)tmp_buff, ISIP_MAX_STRING_LENGTH, fp) != (char*)NULL) {
    
    // get rid of new line
    //
    tmp_buff[strlen((char*)tmp_buff)-1] = ISIP_NULL;

    char_1 tmp_attr[ISIP_MAX_STRING_LENGTH];
    char_1 tmp_class[ISIP_MAX_STRING_LENGTH];
    
    // skipping comment line and/or blank line as well as invalid number
    // of data entry
    //
    if ((tmp_buff[0] != GTREE_COMMENT_CHAR) &&
	(sscanf((char*)tmp_buff, "%[^\t]\t%s", tmp_attr, tmp_class) == 2)) {
      
      // make sure we have enough space for new data
      // 
      if (num_items_a == array_size_a) {
	int_4 tmp_size = array_size_a;
	
	// always return ISIP_TRUE no need to check for error status
	//
	grow_string_arrays_cc(attributes_a, tmp_size, GTREE_ARRAY_GROW_SIZE);
	grow_string_arrays_cc(classes_a, array_size_a, GTREE_ARRAY_GROW_SIZE);
      }
      
      // add the new data
      //
      attributes_a[num_items_a] = new char_1[strlen((char*)tmp_attr)+1];
      strcpy((char*)attributes_a[num_items_a], (char*)tmp_attr);
      classes_a[num_items_a] = new char_1[strlen((char*)tmp_class)+1];
      strcpy((char*)classes_a[num_items_a], (char*)tmp_class);

      if (debug_level_a > ISIP_DEBUG_DETAILED) {
	fprintf(stdout, "%ld %s\t%s\n", num_items_a, attributes_a[num_items_a],
		classes_a[num_items_a]);
      }

      // increment counters
      //
      num_items_a++;
      
    }
    else {
      fprintf(stdout, "skipping....>%s<\n", tmp_buff);
    }
  }
  
  fclose(fp);
  
  // exit gracefully
  //
  return ISIP_TRUE;
}






