// file: $ISIP_PRON/src/decision_tree/v1.0/util/rebuild/r_qsort_0.cc
//

// isip include files
//
#include "rebuild.h"

#include <integral_constants.h>

// method: qsort_cc
//
// arguments:
//  int_4* index_a: (output) buffer containing index array
//  float_4* values_a: (input) buffer containing the data to be sorted
//  int_4 num_items_a: (input) number of items in data buffer
//
// return:
//  logical_1 to indicate status
//
// This methods sorts the given buffer by returning an index array
// that will be used to index into the original array to get the
// sorted data.
//
//
logical_1 qsort_cc(int_4* index_a, float_4* values_a, int_4 num_items_a) {
  
  // copy the data into a temporary space
  //
  float_4 temp[num_items_a];
  memcpy(temp, values_a, num_items_a*sizeof(float_4));
  
  // sort the temp array
  //
  qsort((float*)temp, (int)num_items_a, sizeof(float_4), compare_float_cc);

  // compute an index array using a replacement algorithm
  //
  for (int_4 i = 0; i < num_items_a; i++) {
    for (int_4 j = 0; j < num_items_a; j++) {
      if (temp[i] == values_a[j]) {
	index_a[i] = j;
	temp[i] = ISIP_MINIMUM_FLOAT_4;
	break;
      }
    }
  }


  // print debug data
  //
  //  if (debug_level_d > ISIP_DEBUG_DETAILED) {
  //    fprintf(stdout, "\nSorted data\n");
  // for (int_4 i=0; i<num_items_a; i++) {
  //  fprintf(stdout, "pos: %ld value:%6.2f\n", index_a[i],
  //      values_a[index_a[i]]);
  // }
  //  }

  // exit gracefully
  //
  return ISIP_TRUE;

}

