// file: $ISIP_PRON/src/decision_tree/v1.0/util/rebuild/rebuild.cc
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

// isip include files
//
#include <integral_constants.h>
#include "rebuild_constants.h"

#include "rebuild.h"

// main program
//
main (int_4 argc, char_1* argv[]) {

  int_4 nbest;
  int_4 max;
  int_4 context;
  char_1* fname_in = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* fname_out = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // parse command line arguments
  //
  if (get_parameters_cc(nbest, max, context, fname_in, fname_out, argc, argv)
      != ISIP_TRUE) {
    fprintf(stdout, "<%s: error in command line arguments>\n", argv[0]);
    return ISIP_ERROR;
  }
  
  FILE* fp_in = fopen((char*)fname_in, "r");
  if (fp_in == (FILE*)NULL) {
    fprintf(stdout, "Unable to open file for reading\n");
    return ISIP_ERROR;
  }
  
  FILE* fp_out = fopen((char*)fname_out, "w");
  if (fp_out == (FILE*)NULL) {
    fprintf(stdout, "Unable to open file for writing\n");
    return ISIP_ERROR;
  }
  
  char_1* buff = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* letter = new char_1[R_MAX_WORD*max];
  char_1** phone = new char_1*[R_MAX_WORD*max];
  float_4* score = new float_4[R_MAX_WORD*max];
  int_4 finish = ISIP_FALSE;
  int_4 count=0;
  char_1* nonwhitespace = new char_1[R_MAX_PHONE];

  for (int_4 i=0; i < R_MAX_WORD*max; i++) {
    phone[i] = new char_1[R_MAX_PHONE];
  }
  
  while (!finish) {

    if (fgets((char*)buff, ISIP_MAX_STRING_LENGTH, fp_in) != NULL) {
      buff[strlen((char*)buff)-1] = ISIP_NULL;
      
      // get next context
      //
      if (sscanf((char*)buff, "%1s", nonwhitespace) > 0) {
	if (sscanf((char*)buff, "%c%s%f", &letter[count], phone[count],
		   &score[count]) == 3) {
	  count++;
	}
	else {
	  fprintf(stdout, "bad format\n");
	  return ISIP_ERROR;
	}
      }
      
      // process this word before going to the next word
      //
      else {
	
	// reset count
	//
	int_4 len = count/max;
	count = 0;

	// output the word spelling
	//
	fprintf(fp_out, "* %c", toupper(letter[0]));
	for (int i=1; i<len; i++) {
	  if (letter[(i-1)*max] != ' ') {
	    fprintf(fp_out, "%c", letter[i*max]);
	  }
	  else {
	    fprintf(fp_out, "%c", toupper(letter[i*max]));
	  }
	}
	fprintf(fp_out, "\n");
	
	// there are max**len strings, but limit them to a reasonable value
	//
	int_4 num_strings = (int_4)pow(max, len);
	if (num_strings > R_MAX_STRINGS) {
	  num_strings = R_MAX_STRINGS;
	}
	
	char_1** out_str = new char_1*[num_strings];
	float_4* out_score = new float_4[num_strings];
	int_4 str_count = 0;
	
	// initialize counter indexing
	//
	int_4 *index = new int_4[len];
	for (int_4 i=0; i<len; i++) {
	  index[i] = 0;
	}
	
	char_1* tmp_str = new char_1[ISIP_MAX_STRING_LENGTH];
	float_4 tmp_score = (float_4)0.0;
	logical_1 done = ISIP_FALSE;
	
	while(!done) {
	  
	  // print the current string
	  //
	  tmp_score=0;
	  int_4 l = 0;
	  memset(tmp_str, 0, sizeof(char_1)*ISIP_MAX_STRING_LENGTH);
	  for (l=0; l<len; l++) {
	    strcat((char*)tmp_str, (char*)phone[l*max + index[l]]);
	    tmp_score += log(score[l*max + index[l]]);
	    strcat((char*)tmp_str, " ");
	  }
	  
	  out_str[str_count] = new char_1[strlen((char*)tmp_str)+1];
	  strcpy((char*)out_str[str_count], (char*)tmp_str);
	  out_score[str_count] = tmp_score;
	  str_count++;

	  int_4 i = 0;
	  for(i=len-1; index[i] == max; i--);
	  int_4 j=i+1;
	  do {
	    j--;
	    done = (j < 0) || (str_count == num_strings);
	    
	    index[j]++;
	  } while (index[j] == max);
	  
	  for(int_4 k=j+1; k<l; k++) {
	    index[k] = 0;
	  }
	}
	
	// sort all the strings in decreasing order of scores
	//
	int_4* order = new int_4[num_strings];
	
	qsort_cc(order, out_score, num_strings);
	
	// output only the top nbest strings and their scores to output file
	//
	logical_1 repeat_l = ISIP_FALSE;
	for (int_4 n=0, num_out = 0; (n<num_strings) && (num_out<nbest); n++) {
	  
	  // check if we have already output this pronunciation
	  //
	  repeat_l = ISIP_FALSE;
	  for (int_4 p = 0; p < n; p++) {
	    if (strcmp((char*)out_str[order[n]],
		       (char*)out_str[order[p]]) == 0) {
	      repeat_l = ISIP_TRUE;
	      break;
	    }
	  }
	  if (repeat_l == ISIP_FALSE) {
	    num_out ++;
	    fprintf(fp_out, "%s\tScore: %f\n", out_str[order[n]],
		    out_score[order[n]]);
	  }
	}
	fprintf(fp_out, "\n");
	
	// free memory
	//
	delete index;
	delete order;
	delete out_score;
        for (int_4 m=0; m<str_count; m++) {
          delete [] out_str[m];
        }
	delete [] out_str;
	delete [] tmp_str;
      }
    }
    else {
      finish = 1;
    }
  }

  // free memory
  //
  delete [] fname_in;
  delete [] fname_out;
  for (int_4 m=0; m<R_MAX_WORD*max; m++) {
    delete [] phone[m];
  }
  delete [] phone;
  delete [] buff;
  delete [] letter;
  delete [] nonwhitespace;
  delete score;
  
  // close files
  //
  fclose(fp_in);
  fclose(fp_out);
}
