// file: text_to_speech.c
//
// main functions for tts
//
// According to isip's standards, all functions should reside in a separate
// file from the driver program.  However, due to a static variable
// used in isip_say_string, it's easier to leave the routine in the
// same file as the main program.
//

// isip include files
//
#include "text_to_speech.h"
#include "text_to_speech_constants.h"

// global and static variables
//
extern char *Revision;
static int verbose = 0;

void isip_say_string PROTO((char isip_phone[], char *s));

void
isip_say_string(isip_phone, s)
     char isip_phone[];
     char *s;
{
  darray_t phone;
  darray_init(&phone, sizeof(char), 128);
  xlate_string(s, &phone);
  if (phone.items) {
    strcpy(isip_phone, phone.data);
  }
  darray_free(&phone);
}


int main PROTO((int argc, char *argv[], char *env[]));

int main(argc, argv, env)
     int argc;
     char** argv;
     char** env;
{
  // contain word or phone to be transcribed into pronunciation/speech
  // 
  char s[MAX_STRING_LENGTH];
  
  // contain name of mapping file
  //
  char map_file[MAX_FNAME_LENGTH];
  
  // contain output file if any
  //
  char output_file[MAX_FNAME_LENGTH];
  
  // set defaults for input and output modes
  //
  int pipe_flag = FALSE;
  int audio_flag = FALSE;
  
  // define and set defaults invalid combination in command line arguments
  //
  int word_flag = FALSE;
  int phone_flag = FALSE;
  float sf = TTS_DEFAULT_SF;
  float speed = TTS_DEFAULT_SPEED;
  
  // set default map file
  //
  strcpy(map_file, TTS_DEFAULT_MAP);
  
  // initialize variables
  //
  memset(s, NULL_CHAR, MAX_STRING_LENGTH);
  memset(output_file, NULL_CHAR, MAX_FNAME_LENGTH);
  
  // parse parameters
  //
  if (get_parameters_c(s, map_file, output_file, &pipe_flag, &audio_flag,
		       &word_flag, &phone_flag, &sf, &speed, argc, argv) 
      == FALSE) {
    fprintf(stdout, "error in command line arguments\n", argv[0]);
    return ERROR; 
  }
  
  // load in map file
  //
  if (map_phones_init_c(map_file) == FALSE) {
    fprintf(stdout, "unable to load map file\n");
    return ERROR;
  }
  
  // process either word or phone but not both
  //
  if ((word_flag == TRUE) && (phone_flag == TRUE)) {
    fprintf(stdout, "can not have %s and %s together\n", 
	    TTS_WORD_OPT, TTS_PHONE_OPT);
    return ERROR;
  }
  
  // process word
  //
  if (word_flag == TRUE) {
    if (process_word_c(s, output_file, pipe_flag, audio_flag, sf, speed)
	== FALSE) {
      fprintf(stdout, "unable to process word\n");
    }
  }
  
  // process phone
  //
  else if (phone_flag == TRUE) {
    if (process_phone_c(s, output_file, pipe_flag, audio_flag, sf, speed)
	== FALSE) {
      fprintf(stdout, "unable to process phone\n");
    }
  }
  
  // free map file
  //
  if (map_phones_free_c() == FALSE) {
    fprintf(stdout, "unable to free space for phonetic symbols\n");
    return ERROR;
  }
  
  // exit gracefully
  //
  return NO_ERROR;
}  

// end of file
//
