// file: $DT/util/analyze_dtree/v1.0/analyze_dtree.cc
//

// isip include files
//
#include "analyze_dtree.h"
#include <decision_tree.h>
#include <attribute_node.h>
#include <linked_list.h>
#include <param_file.h>


#include "analyze_dtree_constants.h"
#include <integral_constants.h>

// main program
//
main(int_4 argc, char_1** argv) {
  
  // declare command line arguments
  //
  char_1* param_fname = (char_1*)NULL;
  char_1* attr_fname = (char_1*)NULL;
  char_1* input_fname = (char_1*)NULL;
  char_1* output_fname = (char_1*)NULL;
  
  // declare a debug parameter
  //
  int_4 debug_level = ISIP_DEBUG_NONE;
  
  // parse the command line
  //
  if (get_parameters_cc(param_fname, attr_fname, input_fname, output_fname,
			debug_level, argc, argv) == ISIP_FALSE) {
    fprintf(stdout, "%s: error in command line arguments\n", argv[0]);
    return ISIP_ERROR;
  }

  // check command line arguments before keep going
  // no sense in training the tree and realizing that we don't
  // have a file to save it under
  //
  if (param_fname == (char_1*)NULL) {
    fprintf (stdout, "no parameter file given\n");
    return ISIP_ERROR;
  }

  if (attr_fname == (char_1*)NULL) {
    fprintf (stdout, "no attribute file given\n");
    return ISIP_ERROR;
  }
  
  if (input_fname == (char_1*)NULL) {
    fprintf (stdout, "no input file given\n");
    return ISIP_ERROR;
  }

  if (output_fname == (char_1*)NULL) {
    fprintf (stdout, "no output file given\n");
    return ISIP_ERROR;
  }
  
  // declare objects and the necessary variables
  // 
  Decision_tree dt;

  // set debug level
  //
  if (dt.set_debug_cc(debug_level) != ISIP_TRUE) {
    fprintf(stdout, "unable to set debug level\n");
    return ISIP_ERROR;
  }

  // load in parameter file
  //
  if (dt.load_params_cc(param_fname) != ISIP_TRUE) {
    fprintf(stdout, "%s: invalid parameter file\n", param_fname);
    return ISIP_ERROR;
  }

  dt.debug_cc(stdout, (char_1*)"blah");
  
  // load in attribute file
  //
  if (dt.load_attributes_cc(attr_fname) != ISIP_TRUE) {
    fprintf(stdout, "%s: invalid attribute file\n", attr_fname);
    return ISIP_ERROR;
  }
  //  dt.display_attributes_cc();
  
  // train or test
  //
  if (dt.compute_cc(input_fname) != ISIP_TRUE) {
    fprintf(stdout, "unable to train or test\n");
    return ISIP_ERROR;
  }

  // if mode is train, save the built tree to a file
  //
  int_4 display_mode = dt.get_display_mode_cc();

  FILE* fp_out = fopen(output_fname, "w");
  if (fp_out == (FILE*)NULL) {
    fprintf(stdout, "unable to create file for writing");
    return ISIP_ERROR;
  }
  
  if (dt.display_cc(fp_out, display_mode) != ISIP_TRUE) {
    fprintf(stdout, "unable to write tree for display %s\n", output_fname);
    return ISIP_ERROR;
  }
  
  // display the data information
  //
  //  dt.display_data_cc();
  
  // delete the paramater, attribute, and output file
  //
  if (param_fname != (char_1*)NULL) {
    delete [] param_fname;
  }
  
  if (input_fname != (char_1*)NULL) {
    delete [] input_fname;
  }

  if (output_fname != (char_1*)NULL) {
    delete [] output_fname;
  }
  
  // exit gracefully
  //
  return ISIP_NO_ERROR;
}





